import shutil

import numpy as np
from pathlib import Path
import os
import sys
import glob
from natsort import os_sorted
import scipy.io as spio
import h5py
import matplotlib.pyplot as plt
import pandas as pd

import copy
import time
from whacc import image_tools
import whacc
import platform
import subprocess
from scipy.signal import medfilt, medfilt2d

from tqdm.autonotebook import tqdm


from IPython.utils import io
import pdb

# def tqdm_import_helper():
#     # with io.capture_output() as captured:  # prevent crazy printing
#     # pdb.set_trace()
#     from tqdm.notebook import tqdm
#     try:
#         for k in tqdm(range(1)):
#             pass
#         return True and isnotebook()
#     except:
#         return False


def isnotebook():
    try:
        c = str(get_ipython().__class__)
        shell = get_ipython().__class__.__name__
        if 'colab' in c:
            return True
        elif shell == 'ZMQInteractiveShell':
            return True  # Jupyter notebook or qtconsole
        elif shell == 'TerminalInteractiveShell':
            return False  # Terminal running IPython
        else:
            return False  # Other type (?)
    except NameError:
        return False  # Probably standard Python interpreter


# if tqdm_import_helper():
#     from tqdm.notebook import tqdm
# else:
#     from tqdm import tqdm


def four_class_labels_from_binary(x):
    a = np.asarray(x)
    b = np.asarray([0] + list(np.diff(a)))
    c = a + b
    c[c == -1] = 3
    return c


def print_h5_keys(h5file, return_list=False, do_print=True):
    with h5py.File(h5file, 'r') as h:
        x = copy.deepcopy(list(h.keys()))
        if do_print:
            print_list_with_inds(x)
        if return_list:
            return x


def copy_h5_key_to_another_h5(h5_to_copy_from, h5_to_copy_to, label_string_to_copy_from, label_string_to_copy_to=None):
    if label_string_to_copy_to is None:
        label_string_to_copy_to = label_string_to_copy_from
    with h5py.File(h5_to_copy_from, 'r') as h:
        with h5py.File(h5_to_copy_to, 'r+') as h2:
            try:
                h2[label_string_to_copy_to][:] = h[label_string_to_copy_from][:]
            except:
                h2.create_dataset(label_string_to_copy_to, shape=np.shape(h[label_string_to_copy_from][:]),
                                  data=h[label_string_to_copy_from][:])


def lister_it(in_list, keep_strings='', remove_string=None):
    if len(in_list) == 0:
        print("in_list was empty, returning in_list")
        return in_list

    def index_list_of_strings(in_list2, cmp_string):
        return np.asarray([cmp_string in string for string in in_list2])

    if isinstance(keep_strings, str): keep_strings = [keep_strings]
    if isinstance(remove_string, str): remove_string = [remove_string]

    keep_i = np.asarray([False] * len(in_list))
    for k in keep_strings:
        keep_i = np.vstack((keep_i, index_list_of_strings(in_list, k)))
    keep_i = np.sum(keep_i, axis=0) > 0

    remove_i = np.asarray([True] * len(in_list))
    if remove_string is not None:
        for k in remove_string:
            remove_i = np.vstack((remove_i, np.invert(index_list_of_strings(in_list, k))))
        remove_i = np.product(remove_i, axis=0) > 0

    inds = keep_i * remove_i  # np.invert(remove_i)
    if inds.size <= 0:
        return []
    else:
        out = np.asarray(in_list)[inds]
    return out


# def lister_it(in_list, keep_strings=None, remove_string=None):
#     """
#
#     Parameters
#     ----------
#     in_list : list
#     keep_strings : list
#     remove_string : list
#
#     Returns
#     -------
#
#     """
#     if isinstance(keep_strings, str):
#         keep_strings = [keep_strings]
#     if isinstance(remove_string, str):
#         remove_string = [remove_string]
#
#     if keep_strings is None:
#         new_list = copy.deepcopy(in_list)
#     else:
#         new_list = []
#         for L in in_list:
#             for k in keep_strings:
#                 if k in L:
#                     new_list.append(L)
#
#     if remove_string is None:
#         new_list_2 = copy.deepcopy(in_list)
#     else:
#         new_list_2 = []
#         for L in new_list:
#             for k in remove_string:
#                 if k not in L:
#                     new_list_2.append(L)
#     final_list = intersect_lists([new_list_2, new_list])
#     return final_list


def plot_pole_tracking_max_vals(h5_file):
    with h5py.File(h5_file, 'r') as hf:
        for i, k in enumerate(hf['max_val_stack'][:]):
            plt.plot()


def get_class_info(c, sort_by_type=True, include_underscore_vars=False, return_name_and_type=False, end_prev_len=40):
    names = []
    type_to_print = []
    for k in dir(c):
        if include_underscore_vars is False and k[0] != '_':
            tmp1 = str(type(eval('c.' + k)))
            type_to_print.append(tmp1.split("""'""")[-2])
            names.append(k)
        elif include_underscore_vars:
            tmp1 = str(type(eval('c.' + k)))
            type_to_print.append(tmp1.split("""'""")[-2])
            names.append(k)
    len_space = ' ' * max(len(k) for k in names)
    len_space_type = ' ' * max(len(k) for k in type_to_print)
    if sort_by_type:
        ind_array = np.argsort(type_to_print)
    else:
        ind_array = np.argsort(names)

    for i in ind_array:
        k1 = names[i]
        k2 = type_to_print[i]
        # k3 = str(c[names[i]])
        k3 = str(eval('c.' + names[i]))
        k1 = (k1 + len_space)[:len(len_space)]
        k2 = (k2 + len_space_type)[:len(len_space_type)]
        if len(k3) > end_prev_len:
            k3 = '...' + k3[-end_prev_len:]
        else:
            k3 = '> ' + k3[-end_prev_len:]

        print(k1 + ' type->   ' + k2 + '  ' + k3)
    if return_name_and_type:
        return names, type_to_print


def get_dict_info(c, sort_by_type=True, include_underscore_vars=False, return_name_and_type=False, end_prev_len=40):
    names = []
    type_to_print = []
    for k in c.keys():
        if include_underscore_vars is False and str(k)[0] != '_':
            tmp1 = str(type(c[k]))
            type_to_print.append(tmp1.split("""'""")[-2])
            names.append(str(k))
        elif include_underscore_vars:
            tmp1 = str(type(c[k]))
            type_to_print.append(tmp1.split("""'""")[-2])
            names.append(str(k))
    len_space = ' ' * max(len(k) for k in names)
    len_space_type = ' ' * max(len(k) for k in type_to_print)
    if sort_by_type:
        ind_array = np.argsort(type_to_print)
    else:
        ind_array = np.argsort(names)

    for i in ind_array:
        k1 = names[i]
        k2 = type_to_print[i]
        try:
            k3 = str(c[names[i]])
        except:
            k3 = str(c[float(names[i])])
        k1 = (k1 + len_space)[:len(len_space)]
        k2 = (k2 + len_space_type)[:len(len_space_type)]
        if len(k3) > end_prev_len:
            k3 = '...' + k3[-end_prev_len:]
        else:
            k3 = '> ' + k3[-end_prev_len:]

        print(k1 + ' type->   ' + k2 + '  ' + k3)
    if return_name_and_type:
        return names, type_to_print


def group_consecutives(vals, step=1):
    """

    Parameters
    ----------
    vals :
        
    step :
         (Default value = 1)

    Returns
    -------

    """
    run = []
    run_ind = []
    result = run
    result_ind = run_ind
    expect = None
    for k, v in enumerate(vals):
        if v == expect:
            if not (np.isnan(v)):
                # print(v)
                # print(expect)
                run.append(v)
                run_ind.append(k)
        else:
            if not (np.isnan(v)):
                run = [v]
                run_ind = [k]
                result.append(run)
                result_ind.append(run_ind)
        expect = v + step
    # print(result)
    if result == []:
        pass
    elif result[0] == []:
        result = result[1:]
        result_ind = result_ind[1:]
    return result, result_ind


def get_h5s(base_dir, print_h5_list=True):
    """

    Parameters
    ----------
    base_dir :
        

    Returns
    -------

    """
    H5_file_list = []
    for path in Path(base_dir + os.path.sep).rglob('*.h5'):
        H5_file_list.append(str(path.parent) + os.path.sep + path.name)
    H5_file_list.sort()
    if print_h5_list:
        print_list_with_inds(H5_file_list)
    return H5_file_list


def check_if_file_lists_match(H5_list_LAB, H5_list_IMG):
    """

    Parameters
    ----------
    H5_list_LAB :
        
    H5_list_IMG :
        

    Returns
    -------

    """
    for h5_LAB, h5_IMG in zip(H5_list_LAB, H5_list_IMG):
        try:
            assert h5_IMG.split(os.path.sep)[-1] in h5_LAB
        except:
            print('DO NOT CONTINUE --- some files do not match on your lists try again')
            assert (1 == 0)
    print('yay they all match!')


def print_list_with_inds(list_in):
    """

    Parameters
    ----------
    list_in :
        

    Returns
    -------

    """
    _ = [print(str(i) + ' ' + k.split(os.path.sep)[-1]) for i, k in enumerate(list_in)]


def get_model_list(model_save_dir):
    """

    Parameters
    ----------
    model_save_dir :
        

    Returns
    -------

    """
    print('These are all the models to choose from...')
    model_2_load_all = glob.glob(model_save_dir + '/*.ckpt')
    print_list_with_inds(model_2_load_all)
    return model_2_load_all


def recursive_dir_finder(base_path, search_term):
    """enter base directory and search term to find all the directories in base directory
      with files matching the search_term. output a sorted list of directories.
      e.g. -> recursive_dir_finder('/content/mydropbox/', '*.mp4')

    Parameters
    ----------
    base_path :
        
    search_term :
        

    Returns
    -------

    """
    matching_folders = []
    for root, dirs, files in os.walk(base_path):
        if glob.glob(root + '/' + search_term):
            matching_folders.append(root)
    try:
        matching_folders = os_sorted(matching_folders)
    except:
        matching_folders = sorted(matching_folders)
    return matching_folders


def get_model_list(model_save_dir):
    """

    Parameters
    ----------
    model_save_dir :
        

    Returns
    -------

    """
    print('These are all the models to choose from...')
    model_2_load_all = sorted(glob.glob(model_save_dir + '/*.ckpt'))
    # print(*model_2_load_all, sep = '\n')
    _ = [print(str(i) + ' ' + k.split(os.path.sep)[-1]) for i, k in enumerate(model_2_load_all)]
    return model_2_load_all


def get_files(base_dir, search_term=''):
    """

    Parameters
    ----------
    base_dir :
        
    search_term :
        

    Returnslister_it
    -------

    """
    file_list = []
    for path in Path(base_dir + '/').rglob(search_term):
        file_list.append(str(path.parent) + os.path.sep + path.name)
    file_list.sort()
    return file_list


'''
these below 3 function used to load mat files into dict easily was found and copied directly from 
https://stackoverflow.com/questions/7008608/scipy-io-loadmat-nested-structures-i-e-dictionaries
and contributed by user -mergen 

to the best of my knowledge code found on stackoverflow is under the creative commons license and as such is legal to 
use in my package. contact phillip.maire@gmail.com if you have any questions. 
'''


def loadmat(filename):
    """this function should be called instead of direct spio.loadmat
    as it cures the problem of not properly recovering python dictionaries
    from mat files. It calls the function check keys to cure all entries
    which are still mat-objects

    Parameters
    ----------
    filename :
        

    Returns
    -------

    """
    data = spio.loadmat(filename, struct_as_record=False, squeeze_me=True)
    return _check_keys(data)


def _check_keys(dict):
    """checks if entries in dictionary are mat-objects. If yes
    todict is called to change them to nested dictionaries

    Parameters
    ----------
    dict :
        

    Returns
    -------

    """
    for key in dict:
        if isinstance(dict[key], spio.matlab.mio5_params.mat_struct):
            dict[key] = _todict(dict[key])
    return dict


def _todict(matobj):
    """A recursive function which constructs from matobjects nested dictionaries

    Parameters
    ----------
    matobj :
        

    Returns
    -------

    """
    dict = {}
    for strg in matobj._fieldnames:
        elem = matobj.__dict__[strg]
        if isinstance(elem, spio.matlab.mio5_params.mat_struct):
            dict[strg] = _todict(elem)
        else:
            dict[strg] = elem
    return dict


def get_inds_of_inds(a, return_unique_list=False):
    a2 = []
    for k in a:
        a2.append(list(np.where([k == kk for kk in a])[0]))
    try:
        inds_of_inds = list(np.unique(a2, axis=0))
        for i, k in enumerate(inds_of_inds):
            inds_of_inds[i] = list(k)
    except:
        inds_of_inds = list(np.unique(a2))
    if return_unique_list:
        return inds_of_inds, pd.unique(a)
    else:
        return inds_of_inds


def inds_around_inds(x, N):
    """

    Parameters
    ----------
    x : array
    N : window size

    Returns
    -------
    returns indices of arrays where array >0 with borders of ((N - 1) / 2), so x = [0, 0, 0, 1, 0, 0, 0] and N = 3
    returns [2, 3, 4]
    """
    assert N / 2 != round(N / 2), 'N must be an odd number so that there are equal number of points on each side'
    cumsum = np.cumsum(np.insert(x, 0, 0))
    a = (cumsum[N:] - cumsum[:-N]) / float(N)
    a = np.where(a > 0)[0] + ((N - 1) / 2)
    return a.astype('int')


def loop_segments(frame_num_array, returnaslist=False):
    """

    Parameters
    ----------
    frame_num_array :
    num of frames in each trial in a list
    Returns
    -------
    2 lists with the proper index for pulling those trials out one by one in a for loop
    Examples
    ________
    a3 = [0, 1, 2, 3, 4, 5, 6, 7, 8]
    frame_num_array = [4, 5]
    for i1, i2 in loop_segments(frame_num_array):
        print(a3[i1:i2])

    >>>[0, 1, 2, 3]
    >>>[4, 5, 6, 7, 8]
    """
    frame_num_array = list(frame_num_array)
    frame_num_array = [0] + frame_num_array
    frame_num_array = np.cumsum(frame_num_array)
    frame_num_array = frame_num_array.astype(int)
    if returnaslist:
        return [list(frame_num_array[:-1]), list(frame_num_array[1:])]
    else:
        return zip(list(frame_num_array[:-1]), list(frame_num_array[1:]))


##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_ below programs users will likely not use_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##
##_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*_*##


def _get_human_contacts_(all_h5s):
    """
    just used to get array of contacts, not meant to be used long term
    Parameters
    ----------
    all_h5s :

    Returns
    -------

    """
    h_cont = []
    a = [k.split(os.path.sep)[-1].split('_', maxsplit=1)[-1] for k in all_h5s]
    inds_of_inds, list_of_uniq_files = get_inds_of_inds(a, True)
    for i, k in enumerate(inds_of_inds):
        tmp1 = np.array([])
        for ii, kk in enumerate(k):
            with h5py.File(all_h5s[kk], 'r') as h:
                tmp1 = np.vstack([tmp1, h['labels'][:]]) if tmp1.size else h['labels'][:]
        h_cont.append(tmp1)
    return h_cont, list_of_uniq_files


def create_master_dataset(h5c, all_h5s_imgs, h_cont, borders=80, max_pack_val=100):
    """
    Parameters
    ----------
    h5c : h5 creator class
    all_h5s_imgs : list of h5s with images
    h_cont : a tensor of human contacts people by frames trial H5 files (down right deep)
    borders : for touch 0000011100000 it will find teh 111 in it and get all the bordering areas around it. points are
    unique so 0000011100000 and 0000010100000 will return the same index
    max_pack_val : speeds up the process by transferring data in chunks of this max size instead of building them all up in memory
    it's a max instead of a set value because it can be if the len(IMAGES)%max_pack_val is equal to 0 or 1 it will crash, so I calculate it
    so that it wont crash.

    Returns
    -------create_master_dataset
    saves an H5 file
    Examples
    ________

    all_h5s = utils.get_h5s('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/finished_contacts/')
    all_h5s_imgs = utils.get_h5s('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/H5_data/')
    h_cont = utils._get_human_contacts_(all_h5s)
    h5c = image_tools.h5_iterative_creator('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/test_____.h5',
                                           overwrite_if_file_exists = True,
                                           color_channel = False)
    utils.create_master_dataset(h5c, all_h5s_imgs, h_cont, borders = 80, max_pack_val = 100)
    """
    frame_nums = []
    for k, k2 in zip(all_h5s_imgs, h_cont):
        if len(k2.shape) == 1:
            k2 = np.vstack((k2, k2))
        with h5py.File(k, 'r') as h:
            max_human_label = np.max(k2, axis=0)
            mean_human_label = np.mean(k2, axis=0)
            mean_human_label = (mean_human_label > 0.5) * 1
            b = inds_around_inds(max_human_label, borders * 2 + 1)
            tmp1, _ = group_consecutives(b)
            for tmp2 in tmp1:
                frame_nums.append(len(tmp2))

            pack_every_x = [k for k in np.flip(range(3, max_pack_val + 1)) if len(b) % k >= 2]
            assert pack_every_x, ['chosen H5 file has value of ', len(b), ' and max_pack_val is ', max_pack_val,
                                  ' increase max_pack_val to prevent this error']
            pack_every_x = np.max(pack_every_x)

            # np.max([k for k in np.flip(range(3, 100)) if len(b) % k >= 2])
            new_imgs = np.array([])
            new_labels = np.array([])
            cntr = 0
            for k3 in tqdm(b):
                cntr += 1
                if new_imgs.size:
                    new_imgs = np.concatenate((new_imgs, h['images'][k3][None, :, :, 0]), axis=0)
                    new_labels = np.append(new_labels, max_human_label[k3])
                else:
                    new_imgs = h['images'][k3][None, :, :, 0]
                    new_labels = mean_human_label[k3]
                if cntr >= pack_every_x:  # this makes it ~ 100X faster than stacking up in memory
                    h5c.add_to_h5(new_imgs, new_labels)
                    new_imgs = np.array([])
                    new_labels = np.array([])
                    cntr = 0
            h5c.add_to_h5(new_imgs, new_labels)

    with h5py.File(h5c.h5_full_file_name, 'r+') as h:
        h.create_dataset('frame_nums', shape=np.shape(frame_nums), data=frame_nums)
        h.create_dataset('inds_extracted', shape=np.shape(b), data=b)


def get_time_it(txt):
    """
  Example
  -------
  import re
  import matplotlib.pyplot as plt
  for k in range(8):
    S = 'test '*10**k
    s2 = 'test'
    %time [m.start() for m in re.finditer(S2, S)]

  # then copy it into a string like below

  txt = '''
  CPU times: user 0 ns, sys: 9.05 ms, total: 9.05 ms
  Wall time: 9.01 ms
  CPU times: user 12 µs, sys: 1 µs, total: 13 µs
  Wall time: 15.7 µs
  CPU times: user 48 µs, sys: 3 µs, total: 51 µs
  Wall time: 56.3 µs
  CPU times: user 281 µs, sys: 0 ns, total: 281 µs
  Wall time: 287 µs
  CPU times: user 2.42 ms, sys: 0 ns, total: 2.42 ms
  Wall time: 2.43 ms
  CPU times: user 21.8 ms, sys: 22 µs, total: 21.8 ms
  Wall time: 21.2 ms
  CPU times: user 198 ms, sys: 21.5 ms, total: 219 ms
  Wall time: 214 ms
  CPU times: user 1.83 s, sys: 191 ms, total: 2.02 s
  Wall time: 2.02 s
  '''
  data = get_time_it(txt)
  ax = plt.plot(data[1:])
  plt.yscale('log')
  """
    vars = [k.split('\n')[0] for k in txt.split('Wall time: ')[1:]]
    a = dict()
    a['s'] = 10 ** 0
    a['ms'] = 10 ** -3
    a['µs'] = 10 ** -6
    data = []
    for k in vars:
        units = k.split(' ')[-1]
        data.append(float(k.split(' ')[0]) * a[units])
    return data


def save_what_is_left_of_your_h5_file(H5_file, do_del_and_rename=0):
    tst_cor = []
    with h5py.File(H5_file, 'a') as hf:
        for k in hf.keys():
            if hf.get(k):
                tst_cor.append(0)
            else:
                tst_cor.append(1)
        if any(tst_cor):
            print('Corrupt file found, creating new file')
            H5_fileTMP = H5_file + 'TMP'
            with h5py.File(H5_file + 'TMP', 'w') as hf2:
                for k in hf.keys():
                    if hf.get(k):
                        print('Adding key ' + k + ' to new temp H5 file...')
                        hf2.create_dataset(k, data=hf[k])
                    else:
                        print('***Key ' + k + ' was corrupt, skipping this key...')
                hf2.close()
                hf.close()
            if do_del_and_rename:
                print('Deleting corrupt H5 file')
                os.remove(H5_file)
                print('renaming new h5 file ')
                os.rename(H5_fileTMP, H5_file)
        else:
            print('File is NOT corrupt!')
    print('FINISHED')


def stack_lag_h5_maker(f, f2, buffer=2, shift_to_the_right_by=0):
    """

    Parameters
    ----------
    f : h5 file with SINGLE FRAMES this is meant to be a test program. if used long term I will change this part
    f2 :
    buffer :
    shift_to_the_right_by :

    Returns
    -------

    """
    h5c = image_tools.h5_iterative_creator(f2, overwrite_if_file_exists=True)
    with h5py.File(f, 'r') as h:
        x = h['frame_nums'][:]
        for ii, (k1, k2) in enumerate(tqdm(loop_segments(x), total=len(x))):
            new_imgs = image_tools.stack_imgs_lag(h['images'][k1:k2], buffer=2, shift_to_the_right_by=0)
            h5c.add_to_h5(new_imgs, np.ones(new_imgs.shape[0]) * -1)

    # copy over the other info from the OG h5 file
    copy_h5_key_to_another_h5(f, f2, 'labels', 'labels')
    copy_h5_key_to_another_h5(f, f2, 'frame_nums', 'frame_nums')


def diff_lag_h5_maker(f3):
    """
    need to use the stack_lag_h5_maker first and then send a copy of that into this one again these program are only a temp
    solution, if we use these methods for the main model then I will make using them more fluid and not depend on one another
    Parameters
    ----------
    f3 : the file from stack_lag_h5_maker output

    Returns
    -------
    """
    # change color channel 0 and 1 to diff images from color channel 3 so color channels 0, 1, and 2 are 0-2, 1-2, and 2
    with h5py.File(f3, 'r+') as h:
        for i in tqdm(range(h['images'].shape[0])):
            k = copy.deepcopy(h['images'][i])
            for img_i in range(2):
                k = k.astype(float)
                a = k[:, :, img_i] - k[:, :, -1]
                a = ((a + 255) / 2).astype(np.uint8)
                h['images'][i, :, :, img_i] = a


def expand_single_frame_to_3_color_h5(f, f2):
    h5c = image_tools.h5_iterative_creator(f2, overwrite_if_file_exists=True)
    with h5py.File(f, 'r') as h:
        x = h['frame_nums'][:]
        for ii, (k1, k2) in enumerate(tqdm(loop_segments(x), total=len(x))):
            new_imgs = h['images'][k1:k2]
            new_imgs = np.tile(new_imgs[:, :, :, None], 3)
            h5c.add_to_h5(new_imgs, np.ones(new_imgs.shape[0]) * -1)

    # copy over the other info from the OG h5 file
    copy_over_all_non_image_keys(f, f2)
    # copy_h5_key_to_another_h5(f, f2, 'labels', 'labels')
    # copy_h5_key_to_another_h5(f, f2, 'frame_nums', 'frame_nums')


def copy_over_all_non_image_keys(f, f2):
    k_names = print_h5_keys(f, return_list=True, do_print=False)
    k_names = lister_it(k_names, remove_string='MODEL_')
    k_names = lister_it(k_names, remove_string='images')
    with h5py.File(f, 'r+') as h:
        with h5py.File(f2, 'r+') as h2:
            for kn in k_names:
                try:
                    h2.create_dataset(kn, data=h[kn])
                except:
                    del h2[kn]
                    h2.create_dataset(kn, data=h[kn])
            # try:
            #     copy_h5_key_to_another_h5(f, f2, kn, kn)
            # except:
            #     del h[kn]
            #     # time.sleep(2)
            #     copy_h5_key_to_another_h5(f, f2, kn, kn)
    if 'frame_nums' not in k_names and 'trial_nums_and_frame_nums' in k_names:
        tnfn = image_tools.get_h5_key_and_concatenate([f], 'trial_nums_and_frame_nums')
        with h5py.File(f2, 'r+') as h:
            try:
                h.create_dataset('frame_nums', data=(tnfn[1, :]).astype(int))
            except:
                del h['frame_nums']
                time.sleep(2)
                h.create_dataset('frame_nums', data=(tnfn[1, :]).astype(int))


def force_write_to_h5(h5_file, data, data_name):
    with h5py.File(h5_file, 'r+') as h:
        try:
            h.create_dataset(data_name, data=data)
        except:
            del h[data_name]
            h.create_dataset(data_name, data=data)


def reduce_to_single_frame_from_color(f, f2):
    h5c = image_tools.h5_iterative_creator(f2, overwrite_if_file_exists=True, color_channel=False)
    with h5py.File(f, 'r') as h:
        try:
            x = h['trial_nums_and_frame_nums'][1, :]
        except:
            x = h['frame_nums'][:]
        for ii, (k1, k2) in enumerate(tqdm(loop_segments(x), total=len(x))):
            new_imgs = h['images'][k1:k2][..., -1]
            # new_imgs = np.tile(new_imgs[:, :, :, None], 3)
            h5c.add_to_h5(new_imgs, np.ones(new_imgs.shape[0]) * -1)

    # copy over the other info from the OG h5 file
    copy_over_all_non_image_keys(f, f2)
    # copy_h5_key_to_another_h5(f, f2, 'labels', 'labels')
    # copy_h5_key_to_another_h5(f, f2, 'frame_nums', 'frame_nums')


def make_all_H5_types(base_dir_all_h5s):
    def last_folder(f):
        tmp1 = str(Path(f).parent.absolute())
        return str(Path(tmp1).parent.absolute()) + os.sep

    for f in get_h5s(base_dir_all_h5s):
        base_f = last_folder(f)
        basename = os.path.basename(f)[:-3] + '_regular.h5'
        basedir = base_f + 'regular' + os.sep
        if not os.path.isdir(basedir):
            os.mkdir(basedir)
        f2 = basedir + basename
        expand_single_frame_to_3_color_h5(f, f2)

        basename = os.path.basename(f)[:-3] + '_3lag.h5'
        basedir = base_f + '3lag' + os.sep
        if not os.path.isdir(basedir):
            os.mkdir(basedir)
        f2 = basedir + basename

        stack_lag_h5_maker(f, f2, buffer=2, shift_to_the_right_by=0)

        basename = os.path.basename(f2)[:-3] + '_diff.h5'
        basedir = base_f + '3lag_diff' + os.sep
        if not os.path.isdir(basedir):
            os.mkdir(basedir)
        f3 = basedir + basename
        shutil.copy(f2, f3)
        diff_lag_h5_maker(f3)


def get_all_label_types_from_array(array):
    all_labels = []
    x1 = copy.deepcopy(array)  # [0, 1]- (no touch, touch)
    all_labels.append(x1)

    x2 = four_class_labels_from_binary(x1)  # [0, 1, 2, 3]- (no touch, touch, onset, offset)
    all_labels.append(x2)

    x3 = copy.deepcopy(x2)
    x3[x3 != 2] = 0
    x3[x3 == 2] = 1  # [0, 1]- (not onset, onset)
    all_labels.append(x3)

    x4 = copy.deepcopy(x2)  # [0, 1]- (not offset, offset)
    x4[x4 != 3] = 0
    x4[x4 == 3] = 1
    all_labels.append(x4)

    x5 = copy.deepcopy(x2)  # [0, 1, 2]- (no event, onset, offset)
    x5[x5 == 1] = 0
    x5[x5 == 2] = 1
    x5[x5 == 3] = 2
    all_labels.append(x5)

    x6 = copy.deepcopy(x2)  # [0, 1, 2]- (no event, onset, offset)
    onset_inds = x6[:-1] == 2
    bool_inds_one_after_onset = np.append(False, onset_inds)
    offset_inds = x6[:-1] == 3
    bool_inds_one_after_offset = np.append(False, offset_inds)
    offset_inds = x6 == 3
    x6[bool_inds_one_after_onset] = 3
    x6[offset_inds] = 4
    x6[bool_inds_one_after_offset] = 5
    all_labels.append(x6)

    x7 = copy.deepcopy(x6)
    x7[x7 == 2] = 0
    x7[x7 == 5] = 0
    x7[x7 == 3] = 2
    x7[x7 == 4] = 3
    all_labels.append(x7)

    resort = [5, 1, 4, 0, 3, 2, 6]
    # resort = range(len(resort))
    a_final = []
    for i in resort:
        a_final.append(all_labels[i])

    return np.asarray(a_final)


def make_alt_labels_h5s(base_dir_all_h5s):
    for f in get_h5s(base_dir_all_h5s):
        basename = '_ALT_LABELS.'.join(os.path.basename(f).split('.'))
        basedir = os.sep.join(f.split(os.sep)[:-2]) + os.sep + 'ALT_LABELS' + os.sep
        if not os.path.isdir(basedir):
            os.mkdir(basedir)
        new_h5_name = basedir + basename

        with h5py.File(f, 'r') as h:

            x1 = copy.deepcopy(h['labels'][:])  # [0, 1]- (no touch, touch)

            x2 = four_class_labels_from_binary(x1)  # [0, 1, 2, 3]- (no touch, touch, onset, offset)

            x3 = copy.deepcopy(x2)
            x3[x3 != 2] = 0
            x3[x3 == 2] = 1  # [0, 1]- (not onset, onset)

            x4 = copy.deepcopy(x2)  # [0, 1]- (not offset, offset)
            x4[x4 != 3] = 0
            x4[x4 == 3] = 1

            x5 = copy.deepcopy(x2)  # [0, 1, 2]- (no event, onset, offset)
            x5[x5 == 1] = 0
            x5[x5 == 2] = 1
            x5[x5 == 3] = 2

            x6 = copy.deepcopy(x2)  # [0, 1, 2]- (no event, onset, offset)
            onset_inds = x6[:-1] == 2
            bool_inds_one_after_onset = np.append(False, onset_inds)
            offset_inds = x6[:-1] == 3
            bool_inds_one_after_offset = np.append(False, offset_inds)
            offset_inds = x6 == 3
            x6[bool_inds_one_after_onset] = 3
            x6[offset_inds] = 4
            x6[bool_inds_one_after_offset] = 5

            x7 = copy.deepcopy(x6)
            x7[x7 == 2] = 0
            x7[x7 == 5] = 0
            x7[x7 == 3] = 2
            x7[x7 == 4] = 3

        with h5py.File(new_h5_name, 'w') as h:
            h.create_dataset('[0, 1]- (no touch, touch)', shape=np.shape(x1), data=x1)
            h.create_dataset('[0, 1, 2, 3]- (no touch, touch, onset, offset', shape=np.shape(x2), data=x2)
            h.create_dataset('[0, 1]- (not onset, onset)', shape=np.shape(x3), data=x3)
            h.create_dataset('[0, 1]- (not offset, offset)', shape=np.shape(x4), data=x4)
            h.create_dataset('[0, 1, 2]- (no event, onset, offset)', shape=np.shape(x5), data=x5)
            h.create_dataset('[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)',
                             shape=np.shape(x6), data=x6)
            h.create_dataset('[0, 1, 2, 3]- (no touch, touch, one after onset, offset)', shape=np.shape(x7), data=x7)


def intersect_lists(d):
    return list(set(d[0]).intersection(*d))


def get_in_range(H5_list, pole_up_add=200, pole_down_add=0, write_to_h5=True, return_in_range=False):
    all_in_range = []
    for k in H5_list:
        with h5py.File(k, 'r+') as hf:
            new_in_range = np.zeros_like(hf['in_range'][:])
            fn = hf['trial_nums_and_frame_nums'][1, :]
            for i, (i1, i2) in enumerate(loop_segments(fn)):
                x = hf['pole_times'][:, i] + i1
                x1 = x[0] + pole_up_add
                x2 = x[1] + pole_down_add
                x2 = min([x2, i2])
                new_in_range[x1:x2] = 1
            if write_to_h5:
                hf['in_range'][:] = new_in_range
            if return_in_range:
                all_in_range.append(new_in_range)
    if return_in_range:
        return all_in_range


def define_in_range(h5_file, pole_up_set_time=0, pole_down_add_to_trigger=0, write_to_h5=True, return_in_range=False):
    with h5py.File(h5_file, 'r+') as hf:
        new_in_range = np.zeros_like(hf['in_range'][:])
        fn = hf['trial_nums_and_frame_nums'][1, :]
        for i, (i1, i2) in enumerate(loop_segments(fn)):
            x = hf['pole_times'][:, i] + i1
            x1 = i1 + pole_up_set_time
            x2 = x[1] + pole_down_add_to_trigger
            x2 = min([x2, i2])
            new_in_range[x1:x2] = 1
        if write_to_h5:
            hf['in_range'][:] = new_in_range
        if return_in_range:
            return new_in_range


def add_to_h5(h5_file, key, values, overwrite_if_exists=False):
    all_keys = print_h5_keys(h5_file, return_list=True, do_print=False)
    with h5py.File(h5_file, 'r+') as h:
        if key in all_keys and overwrite_if_exists:
            print('key already exists, overwriting value...')
            del h[key]
            h.create_dataset(key, data=values)
        elif key in all_keys and not overwrite_if_exists:
            print("""key already exists, NOT overwriting value..., \nset 'overwrite_if_exists' to True to overwrite""")
        else:
            h.create_dataset(key, data=values)


def bool_pred_to_class_pred_formating(pred):
    pred = pred.flatten()
    zero_pred = np.ones_like(pred) - pred
    x = np.vstack((zero_pred, pred)).T
    return x


def convert_labels_back_to_binary(b, key):
    a = copy.deepcopy(b)
    """
  a is 'bool' array (integers not float predicitons)
  key is the key name of the type of labels being inserted
  can use the key name or the string in the key either will do.
  """
    name_dict = whacc.model_maker.label_naming_shorthand_dict()
    keys = list(name_dict.keys())
    if key == keys[0] or key == name_dict[keys[0]]:
        a[a >= 4] = 0
        a[a >= 2] = 1

    elif key == keys[1] or key == name_dict[keys[1]]:
        a[a >= 3] = 0
        a[a >= 2] = 1

    elif key == keys[2] or key == name_dict[keys[2]]:
        print("""can not convert type """ + key + ' returning None')
        return None
    elif key == keys[3] or key == name_dict[keys[3]]:
        print('already in the correct format')
    elif key == keys[4] or key == name_dict[keys[4]]:
        print("""can not convert type """ + key + ' returning None')
        return None
    elif key == keys[5] or key == name_dict[keys[5]]:
        print("""can not convert type """ + key + ' returning None')
        return None
    elif key == keys[6] or key == name_dict[keys[6]]:
        a[a >= 3] = 0
        one_to_the_left_inds = a == 2
        one_to_the_left_inds = np.append(one_to_the_left_inds[1:], False)
        a[one_to_the_left_inds] = 1
        a[a == 2] = 1
    else:
        raise ValueError("""key does not match. invalid key --> """ + key)
    return a


def update_whacc():
    x = '''python3 "/Users/phil/Dropbox/UPDATE_WHACC_PYPI.py"'''
    out = os.popen(x).read()
    print(out)
    print('ALL DONE')


def make_list(x, suppress_warning=False):
    if not isinstance(x, list):
        if not suppress_warning:
            print("""input is supposed to be a list, converting it but user should do this to suppress this warning""")
        x2 = [x]
        return x2
    else:
        return x


def search_sequence_numpy(arr, seq, return_type='indices'):
    Na, Nseq = arr.size, seq.size
    r_seq = np.arange(Nseq)
    M = (arr[np.arange(Na - Nseq + 1)[:, None] + r_seq] == seq).all(1)

    if return_type == 'indices':
        return np.where(M)[0]
    elif return_type == 'bool':
        return M


def find_trials_with_suspicious_predictions(frame_nums, pred_bool, tmp_weights=[3, 3, 2, 1]):
    all_lens = []
    bins = len(tmp_weights) + 1
    for i, (k1, k2) in enumerate(loop_segments(frame_nums)):
        vals = pred_bool[k1:k2]
        a, b = group_consecutives(vals, step=0)
        y, x = np.histogram([len(k) for k in a], np.linspace(1, bins, bins))
        all_lens.append(y)
    all_lens = np.asarray(all_lens)

    all_lens = all_lens * np.asarray(tmp_weights)
    sorted_worst_estimated_trials = np.flip(np.argsort(np.nanmean(all_lens, axis=1)))
    return sorted_worst_estimated_trials


def open_folder(path):
    if platform.system() == "Windows":
        os.startfile(path)
    elif platform.system() == "Darwin":
        subprocess.Popen(["open", path])
    else:
        subprocess.Popen(["xdg-open", path])


def medfilt_confidence_scores(pred_bool_in, kernel_size_in):
    if pred_bool_in.shape[1] == 1:
        pred_bool_out = medfilt(copy.deepcopy(pred_bool_in), kernel_size=kernel_size_in)
    else:
        pred_bool_out = medfilt2d(copy.deepcopy(pred_bool_in), kernel_size=[kernel_size_in, 1])
    return pred_bool_out


def confidence_score_to_class(pred_bool_in, thresh_in=0.5):
    if pred_bool_in.shape[1] == 1:
        pred_bool_out = ((pred_bool_in > thresh_in) * 1).flatten()
    else:
        pred_bool_out = np.argmax(pred_bool_in, axis=1)
    #     NOTE: threshold is not used for the multi class models
    return pred_bool_out


def process_confidence_scores(pred_bool_in, key_name_in, thresh_in=0.5, kernel_size_in=1):
    pred_bool_out = medfilt_confidence_scores(pred_bool_in, kernel_size_in)
    pred_bool_out = confidence_score_to_class(pred_bool_out, thresh_in)
    L_key_ = '_'.join(key_name_in.split('__')[3].split(' '))
    pred_bool_out = convert_labels_back_to_binary(pred_bool_out, L_key_)
    return pred_bool_out


def copy_folder_structure(src, dst):
    src = os.path.abspath(src)
    src_prefix = len(src) + len(os.path.sep)
    Path(dst).mkdir(parents=True, exist_ok=True)
    for root, dirs, files in os.walk(src):
        for dirname in dirs:
            dirpath = os.path.join(dst, root[src_prefix:], dirname)
            Path(dirpath).mkdir(parents=True, exist_ok=True)


def copy_file_filter(src, dst, keep_strings='', remove_string=None, overwrite=False,
                     just_print_what_will_be_copied=False, disable_tqdm=False, return_list_of_files=False):
    """

    Parameters
    ----------
    return_list_of_files :
    src : source folder
    dst : destination folder
    keep_strings : see utils.lister_it, list of strings to match in order to copy
    remove_string : see utils.lister_it, list of strings to match in order to not copy
    overwrite : will overwrite files if true
    just_print_what_will_be_copied : can just print what will be copied to be sure it is correct
    disable_tqdm : if True it will prevent the TQDM loading bar

    Examples
    ________
    copy_file_filter('/Users/phil/Desktop/FAKE_full_data', '/Users/phil/Desktop/aaaaaaaaaa', keep_strings='/3lag/',
                 remove_string=None, overwrite=True, just_print_what_will_be_copied=False)
    Returns
    -------

    """
    src = src.rstrip(os.sep) + os.sep
    dst = dst.rstrip(os.sep) + os.sep

    all_files_and_dirs = get_files(src, search_term='*')
    to_copy = lister_it(all_files_and_dirs, keep_strings=keep_strings, remove_string=remove_string)

    if just_print_what_will_be_copied:
        _ = [print(str(i) + ' ' + k) for i, k in enumerate(to_copy)]
        if return_list_of_files:
            return to_copy, None
        else:
            return

    to_copy2 = []  # this is so I can tqdm the files and not the folders which would screw with the average copy time.
    for k in to_copy:
        k2 = dst.join(k.split(src))
        if os.path.isdir(k):
            Path(k2).mkdir(parents=True, exist_ok=True)
        else:
            to_copy2.append(k)
    final_copied = []
    for k in tqdm(to_copy2, disable=disable_tqdm):
        k2 = dst.join(k.split(src))
        final_copied.append(k2)
        if overwrite or not os.path.isfile(k2):
            if os.path.isfile(k2):
                os.remove(k2)
            Path(os.path.dirname(k2)).mkdir(parents=True, exist_ok=True)
            shutil.copyfile(k, k2)
        elif not overwrite:
            print('overwrite = False: file exists, skipping--> ' + k2)
    if return_list_of_files:
        return to_copy2, final_copied


def copy_alt_labels_based_on_directory(file_list, alt_label_folder_name='ALT_LABELS'):
    h5_imgs = []
    h5_labels = []
    inds_of_files = []
    for i, k in enumerate(file_list):
        k = k.rstrip(os.sep)
        if os.path.isfile(k):
            fn = os.path.basename(k)
            alt_labels_dir = os.sep.join(k.split(os.sep)[:-2]) + os.sep + alt_label_folder_name + os.sep
            h5_list = get_h5s(alt_labels_dir, 0)
            if 'train' in fn.lower():
                h5_list = lister_it(h5_list, keep_strings='train')
            elif 'val' in fn.lower():
                h5_list = lister_it(h5_list, keep_strings='val')
            if len(h5_list) == 1:
                h5_imgs.append(k)
                h5_labels.append(h5_list[0])
                inds_of_files.append(i)
            else:
                print('File name ' + fn + ' could not find valid match')
                break
    return h5_imgs, h5_labels, inds_of_files


def np_stats(in_arr):
    print('\nmin', np.min(in_arr))
    print('max', np.max(in_arr))
    print('mean', np.mean(in_arr))
    print('shape', in_arr.shape)
    print('len of unique', len(np.unique(in_arr)))
    print('type', type(in_arr))
    try:
        print('Dtype ', in_arr.dtype)
    except:
        pass


def h5_key_exists(h5_in, key_in):
    return key_in in print_h5_keys(h5_in, return_list=True, do_print=False)



# get_ipython().system('pip install whacc -U')


from tqdm.notebook import tqdm
from tqdm import tqdm
import pickle
from whacc import utils, image_tools, transfer_learning, analysis
import matplotlib.pyplot as plt

from sklearn.utils import class_weight
import os
import copy
import numpy as np

from pathlib import Path
import shutil
import zipfile
import pytz
import json
from math import isclose, sqrt
from IPython import display

import pandas as pd
import time

# import lightgbm as lgb
# from datetime import datetime

import datetime
import h5py



def shift_with_frame_nums(data_frame_in, frame_nums, shift_by, add_name_str = None):
    if add_name_str is None:
        add_name_str = 'shifted_'+str(shift_by)
    data_frame = data_frame_in.copy()
    for i1, i2 in utils.loop_segments(frame_nums):
        data_frame[i1:i2] = data_frame_in[i1:i2].shift(shift_by)
    data_frame = data_frame.add_suffix('_'+add_name_str)
    return data_frame

def rolling_mean_with_frame_nums(data_frame_in, frame_nums, window, add_name_str=None):
    if add_name_str is None:
        add_name_str = 'rolling_mean_' + str(window)
    data_frame = data_frame_in.copy()
    for i1, i2 in utils.loop_segments(frame_nums):
        df_rolling = data_frame_in[i1:i2].rolling(window=window, min_periods=window)
        data_frame[i1:i2] = df_rolling.mean().shift(-(window // 2)).astype(np.float32)
    data_frame = data_frame.add_suffix('_' + add_name_str)
    return data_frame


def rolling_std_with_frame_nums(data_frame_in, frame_nums, window, add_name_str=None):
    if add_name_str is None:
        add_name_str = 'rolling_std_' + str(window)
    data_frame = data_frame_in.copy()
    for i1, i2 in utils.loop_segments(frame_nums):
        df_rolling = data_frame_in[i1:i2].rolling(window=window, min_periods=window)
        data_frame[i1:i2] = df_rolling.std().shift(-(window // 2)).astype(np.float32)
    data_frame = data_frame.add_suffix('_' + add_name_str)
    return data_frame


def concat_numpy_memmory_save(df__x, base_dir):
    np_list = utils.get_files(base_dir + os.sep, '*.npy')
    init_shape = list(df__x.shape)
    s = init_shape[1]
    init_shape[1] = s * len(np_list) + s
    final_x = np.zeros(init_shape, dtype=np.float32)

    final_x[:, :s] = df__x.astype('float32')
    for i, f in tqdm(enumerate(np_list)):
        ii = i + 1
        i1 = s * ii
        i2 = s * ii + s
        final_x[:, i1:i2] = np.load(f, allow_pickle=True).astype('float32')
    return final_x


def transform_x_data_np(df_in, frame_nums_in, base_dir):
    if os.path.isdir(base_dir):
        print("WARNING WARNING WARNING WARNING directory already exists please delete directory to redo, returning")
        return
    save_dir = base_dir + os.sep
    Path(save_dir).mkdir(parents=True, exist_ok=True)
    st = time.time()
    cnt = 0
    for shift_by in tqdm([-2, -1, 1, 2]):
        x = shift_with_frame_nums(df_in, frame_nums_in, shift_by).to_numpy()
        np.save(save_dir + str(cnt) + '.npy', x)
        cnt += 1
    for roll_by in tqdm([3, 7]):
        x = rolling_mean_with_frame_nums(df_in, frame_nums_in, roll_by)
        np.save(save_dir + str(cnt) + '.npy', x)
        cnt += 1
        x = rolling_std_with_frame_nums(df_in, frame_nums_in, roll_by)
        np.save(save_dir + str(cnt) + '.npy', x)
        cnt += 1
    print(time.time() - st)


def make_data_chunks(h5, base_dir, do_transform_data=True, label_key='[0, 1]- (no touch, touch)'):
    time_length = 1  # do not change we shift time in pandas dataframe now
    batch_size = None
    feature_len = 2048
    label_index_to_lstm_len = None
    edge_value = np.nan
    remove_any_time_points_with_edges = False
    #     'labels'
    h5 = utils.make_list(h5)
    test_x = None
    for tmp_h5 in h5:

        # if label_key not in utils.print_h5_keys(tmp_h5, return_list=True, do_print=False):
        #     label_key='labels'
        G_test = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, tmp_h5, label_key=label_key,
                                                               feature_len=feature_len,
                                                               label_index_to_lstm_len=label_index_to_lstm_len,
                                                               edge_value=edge_value,
                                                               remove_any_time_points_with_edges=remove_any_time_points_with_edges)
        tmp_frame_nums_test = image_tools.get_h5_key_and_concatenate(tmp_h5, 'frame_nums')
        tmpx, tmpy = G_test.__getitem__(0)

        print(tmpx.shape, tmpy.shape)
        if test_x is not None:
            test_x = np.vstack((test_x, tmpx))
            test_y = np.concatenate((test_y, tmpy.flatten()))
            frame_nums_test = np.concatenate((frame_nums_test, tmp_frame_nums_test.flatten()))

        else:
            test_x = tmpx
            test_y = tmpy.flatten()
            frame_nums_test = tmp_frame_nums_test.flatten()

    df_test_x = pd.DataFrame(test_x)

    del test_x  # save RAM
    if do_transform_data:
        transform_x_data_np(df_test_x, frame_nums_test, base_dir)
    return df_test_x, test_y


####


def foo_rename(instr):
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        tmp1 = instr.split('My Drive')[-1]
        a = tmp1[0]
        a2 = os.sep
        x = a + 'Volumes' + a + 'GoogleDrive-114825029448473821206' + a + 'My Drive' + tmp1
        x = a2.join(x.split(a))
        return x


base_dir_all = r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\DATA_FULL'
base_dir_all = foo_rename(base_dir_all)
base_dir_all
os.path.isdir(base_dir_all)

h5 = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'
utils.print_h5_keys(h5)
with h5py.File(h5, 'r') as h:
    tmp1 = h['images'][:20]
    print(h['model_name_used_as_feature_extractor'][0])

mod_name = 'MODEL_3_regular 80 border aug 0 to 9__ResNet50V2__3lag__regular__acc_test max__epoch 6__L_ind3__LABELS_-_2021_07_22_06_01_29'
"""ADD MODEL USED TO GET FEATURES FOR REPEATABILITY """
with h5py.File(h5, 'r+') as h:
    dname = 'model_name_used_as_feature_extractor_' + mod_name
    h.create_dataset(dname, data=0)

with h5py.File(h5, 'r+') as h:
    h.create_dataset_like('images')

#
def shift_with_frame_nums(data_frame_in, frame_nums, shift_by, add_name_str = None):
    if add_name_str is None:
        add_name_str = 'shifted_'+str(shift_by)
    data_frame = data_frame_in.copy()
    for i1, i2 in utils.loop_segments(frame_nums):
        data_frame[i1:i2] = data_frame_in[i1:i2].shift(shift_by)
    data_frame = data_frame.add_suffix('_'+add_name_str)
    return data_frame


h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'
shift_by = -1
add_name_str = None





class feature_maker():
    def __init__(self, h5_in, frame_nums = None, operational_key = 'images'):
        self.h5_in = h5_in
        if frame_nums is None:
            self.frame_nums = image_tools.get_h5_key_and_concatenate(h5_in, 'frame_nums')
        self.operational_key = operational_key
        self.set_operation_key()

        # self.xxxx = 11111
        # self.xxxx = 11111
        # self.xxxx = 11111
        # self.xxxx = 11111
        # self.xxxx = 11111
        # self.xxxx = 11111
    def set_operation_key(self, key_name = None):
        if key_name is not None:
            self.operational_key = key_name
        self.data = pd.DataFrame(image_tools.get_h5_key_and_concatenate(h5_in, self.operational_key))


    # def _rolling_(self, window, shift_from_center = 0, min_periods = None):
    #     if min_periods is None:
    #         min_periods = window
    #     add_name_list = ['FD__' + self.operational_key + 'rolling_',   '_W_' + str(window) + '_SFC_' + str(shift_from_center) + '_MP_' + str(min_periods)]
    #     data_frame = self.data.copy()
    #     return window, shift_from_center, min_periods, add_name_list, data_frame
    def rolling(self, window, operation, shift_from_center = 0, min_periods = None):
        if min_periods is None:
            min_periods = window
        add_name_list = ['FD__' + self.operational_key + '_rolling_',   '_W_' + str(window) + '_SFC_' + str(shift_from_center) + '_MP_' + str(min_periods)]
        data_frame = self.data.copy()
        add_name_str = operation.join(add_name_list)
        for i1, i2 in utils.loop_segments(self.frame_nums):
            df_rolling = self.data[i1:i2].rolling(window=window, min_periods=min_periods, center=True)
            tmp_func = eval('df_rolling.' + operation)
            data_frame[i1:i2] = tmp_func().shift(shift_from_center).astype(np.float32)
        return data_frame, add_name_str
    def shift(self, shift_by):
        add_name_str = 'FD__' + self.operational_key + '_shift_' + str(shift_by)
        data_frame = self.data.copy()
        for i1, i2 in utils.loop_segments(self.frame_nums):
            data_frame[i1:i2] = self.data[i1:i2].shift(shift_by).astype(np.float32)
        return data_frame, add_name_str

h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'
feature_data = image_tools.get_h5_key_and_concatenate(h5_in, 'images')
feature_data = pd.DataFrame(feature_data)


frame_nums = image_tools.get_h5_key_and_concatenate(h5_in, 'frame_nums')
frame_class = np.zeros(np.sum(frame_nums)).astype(int)
for i, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
    frame_class[i1:i2] = i
frame_class


feature_data['frame_nums'] =frame_class


grouped = feature_data.groupby('frame_# get_ipython().system('pip install whacc -U')


from tqdm.autonotebook import tqdm
from tqdm import tqdm
import pickle
from whacc import utils, image_tools, transfer_learning, analysis
import matplotlib.pyplot as plt

from sklearn.utils import class_weight
import os
import copy
import numpy as np

from pathlib import Path
import shutil
import zipfile
import pytz
import json
from math import isclose, sqrt
from IPython import display

import pandas as pd
import time

# import lightgbm as lgb
# from datetime import datetime

import datetime
import h5py



def shift_with_frame_nums(data_frame_in, frame_nums, shift_by, add_name_str = None):
    if add_name_str is None:
        add_name_str = 'shifted_'+str(shift_by)
    data_frame = data_frame_in.copy()
    for i1, i2 in utils.loop_segments(frame_nums):
        data_frame[i1:i2] = data_frame_in[i1:i2].shift(shift_by)
    data_frame = data_frame.add_suffix('_'+add_name_str)
    return data_frame

def rolling_mean_with_frame_nums(data_frame_in, frame_nums, window, add_name_str=None):
    if add_name_str is None:
        add_name_str = 'rolling_mean_' + str(window)
    data_frame = data_frame_in.copy()
    for i1, i2 in utils.loop_segments(frame_nums):
        df_rolling = data_frame_in[i1:i2].rolling(window=window, min_periods=window)
        data_frame[i1:i2] = df_rolling.mean().shift(-(window // 2)).astype(np.float32)
    data_frame = data_frame.add_suffix('_' + add_name_str)
    return data_frame


def rolling_std_with_frame_nums(data_frame_in, frame_nums, window, add_name_str=None):
    if add_name_str is None:
        add_name_str = 'rolling_std_' + str(window)
    data_frame = data_frame_in.copy()
    for i1, i2 in utils.loop_segments(frame_nums):
        df_rolling = data_frame_in[i1:i2].rolling(window=window, min_periods=window)
        data_frame[i1:i2] = df_rolling.std().shift(-(window // 2)).astype(np.float32)
    data_frame = data_frame.add_suffix('_' + add_name_str)
    return data_frame


def concat_numpy_memmory_save(df__x, base_dir):
    np_list = utils.get_files(base_dir + os.sep, '*.npy')
    init_shape = list(df__x.shape)
    s = init_shape[1]
    init_shape[1] = s * len(np_list) + s
    final_x = np.zeros(init_shape, dtype=np.float32)

    final_x[:, :s] = df__x.astype('float32')
    for i, f in tqdm(enumerate(np_list)):
        ii = i + 1
        i1 = s * ii
        i2 = s * ii + s
        final_x[:, i1:i2] = np.load(f, allow_pickle=True).astype('float32')
    return final_x


def transform_x_data_np(df_in, frame_nums_in, base_dir):
    if os.path.isdir(base_dir):
        print("WARNING WARNING WARNING WARNING directory already exists please delete directory to redo, returning")
        return
    save_dir = base_dir + os.sep
    Path(save_dir).mkdir(parents=True, exist_ok=True)
    st = time.time()
    cnt = 0
    for shift_by in tqdm([-2, -1, 1, 2]):
        x = shift_with_frame_nums(df_in, frame_nums_in, shift_by).to_numpy()
        np.save(save_dir + str(cnt) + '.npy', x)
        cnt += 1
    for roll_by in tqdm([3, 7]):
        x = rolling_mean_with_frame_nums(df_in, frame_nums_in, roll_by)
        np.save(save_dir + str(cnt) + '.npy', x)
        cnt += 1
        x = rolling_std_with_frame_nums(df_in, frame_nums_in, roll_by)
        np.save(save_dir + str(cnt) + '.npy', x)
        cnt += 1
    print(time.time() - st)


def make_data_chunks(h5, base_dir, do_transform_data=True, label_key='[0, 1]- (no touch, touch)'):
    time_length = 1  # do not change we shift time in pandas dataframe now
    batch_size = None
    feature_len = 2048
    label_index_to_lstm_len = None
    edge_value = np.nan
    remove_any_time_points_with_edges = False
    #     'labels'
    h5 = utils.make_list(h5)
    test_x = None
    for tmp_h5 in h5:

        # if label_key not in utils.print_h5_keys(tmp_h5, return_list=True, do_print=False):
        #     label_key='labels'
        G_test = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, tmp_h5, label_key=label_key,
                                                               feature_len=feature_len,
                                                               label_index_to_lstm_len=label_index_to_lstm_len,
                                                               edge_value=edge_value,
                                                               remove_any_time_points_with_edges=remove_any_time_points_with_edges)
        tmp_frame_nums_test = image_tools.get_h5_key_and_concatenate(tmp_h5, 'frame_nums')
        tmpx, tmpy = G_test.__getitem__(0)

        print(tmpx.shape, tmpy.shape)
        if test_x is not None:
            test_x = np.vstack((test_x, tmpx))
            test_y = np.concatenate((test_y, tmpy.flatten()))
            frame_nums_test = np.concatenate((frame_nums_test, tmp_frame_nums_test.flatten()))

        else:
            test_x = tmpx
            test_y = tmpy.flatten()
            frame_nums_test = tmp_frame_nums_test.flatten()

    df_test_x = pd.DataFrame(test_x)

    del test_x  # save RAM
    if do_transform_data:
        transform_x_data_np(df_test_x, frame_nums_test, base_dir)
    return df_test_x, test_y


####


def foo_rename(instr):
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        tmp1 = instr.split('My Drive')[-1]
        a = tmp1[0]
        a2 = os.sep
        x = a + 'Volumes' + a + 'GoogleDrive-114825029448473821206' + a + 'My Drive' + tmp1
        x = a2.join(x.split(a))
        return x


base_dir_all = r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\DATA_FULL'
base_dir_all = foo_rename(base_dir_all)
base_dir_all
os.path.isdir(base_dir_all)

h5 = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'
utils.print_h5_keys(h5)
with h5py.File(h5, 'r') as h:
    tmp1 = h['images'][:20]
    print(h['model_name_used_as_feature_extractor'][0])

mod_name = 'MODEL_3_regular 80 border aug 0 to 9__ResNet50V2__3lag__regular__acc_test max__epoch 6__L_ind3__LABELS_-_2021_07_22_06_01_29'
"""ADD MODEL USED TO GET FEATURES FOR REPEATABILITY """
with h5py.File(h5, 'r+') as h:
    dname = 'model_name_used_as_feature_extractor_' + mod_name
    h.create_dataset(dname, data=0)

with h5py.File(h5, 'r+') as h:
    h.create_dataset_like('images')

#
def shift_with_frame_nums(data_frame_in, frame_nums, shift_by, add_name_str = None):
    if add_name_str is None:
        add_name_str = 'shifted_'+str(shift_by)
    data_frame = data_frame_in.copy()
    for i1, i2 in utils.loop_segments(frame_nums):
        data_frame[i1:i2] = data_frame_in[i1:i2].shift(shift_by)
    data_frame = data_frame.add_suffix('_'+add_name_str)
    return data_frame


h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'
shift_by = -1
add_name_str = None




"""
could use with h5py instead of get_and_concat and then create the class for each set of frame nums, basically make a 
wrapper class that wraps this class
"""

class feature_maker():
    def __init__(self, h5_in, frame_num_ind = None, frame_nums = None, operational_key = 'images'):
        self.h5_in = h5_in
        self.frame_num_ind = frame_num_ind
        if frame_nums is None:
            with h5py.File(self.h5_in, 'r') as h:
                if self.frame_num_ind is None:
                    self.frame_nums = copy.deepcopy(h['frame_nums'][:])
                else:
                    self.frame_nums = copy.deepcopy([h['frame_nums'][self.frame_num_ind]])
                    tmp_inds = np.asarray(utils.loop_segments(self.frame_nums, returnaslist=True))
                    self.data_inds = tmp_inds[:, self.frame_num_ind]
        self.operational_key = operational_key
        self.set_operation_key(operational_key)
    def set_operation_key(self, key_name):
        self.operational_key = key_name
        with h5py.File(self.h5_in, 'r') as h:
            if self.frame_num_ind is None:
                self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][:]))
            else:
                a = self.data_inds
                self.data = pd.DataFrame(copy.deepcopy(h[self.operational_key][a[0]:a[1]]))

    def rolling(self, window, operation, shift_from_center = 0, min_periods = None, kwargs = {}):
        if min_periods is None:
            min_periods = window
        add_name_list = ['FD__' + self.operational_key + '_rolling_',
                         '_W_' + str(window) + '_SFC_' + str(shift_from_center) + '_MP_' + str(min_periods)]
        data_frame = self.data.copy()
        add_name_str = operation.join(add_name_list)
        for i1, i2 in utils.loop_segments(self.frame_nums):
            df_rolling = self.data[i1:i2].rolling(window=window, min_periods=min_periods, center=True)
            tmp_func = eval('df_rolling.' + operation)
            data_frame[i1:i2] = tmp_func(**kwargs).shift(shift_from_center).astype(np.float32)
        return data_frame, add_name_str
    def shift(self, shift_by):
        add_name_str = 'FD__' + self.operational_key + '_shift_' + str(shift_by)
        data_frame = self.data.copy()
        for i1, i2 in utils.loop_segments(self.frame_nums):
            data_frame[i1:i2] = self.data[i1:i2].shift(shift_by).astype(np.float32)
        return data_frame, add_name_str

    def operate(self, operation, add_name_str = '', kwargs = {}):
        add_name_str = 'FD__' + self.operational_key + '_'+operation+'_' + add_name_str
        data_frame = self.data.copy()
        for i1, i2 in utils.loop_segments(self.frame_nums):
            data_frame[i1:i2] = eval('self.data[i1:i2].' + operation + '(**kwargs).astype(np.float32)')
        return data_frame, add_name_str


h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'

FM = feature_maker(h5_in, frame_num_ind = 0)

# tmp1, tmp2 = FM.shift(2)
# tmp1 = pd.DataFrame(np.random.rand(10, 5))
kwargs = {'periods':-1}
data_frame, add_name_str = FM.operate('shift', kwargs = kwargs)

kargs = {'min_periods':0, 'center':True}
kargs = {'periods':2}
tmp1, tmp2 = FM.rolling(7, 'mean')



#
#
# h5_in = '/Users/phil/Desktop/AH0407_160613_JC1003_AAAC_3lag.h5'
# feature_data = image_tools.get_h5_key_and_concatenate(h5_in, 'images')
# feature_data = pd.DataFrame(feature_data)
#
#
# frame_nums = image_tools.get_h5_key_and_concatenate(h5_in, 'frame_nums')
# frame_class = np.zeros(np.sum(frame_nums)).astype(int)
# for i, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
#     frame_class[i1:i2] = i
# frame_class
#
#
# feature_data['frame_nums'] =frame_class
#
#
# grouped = feature_data.groupby('frame_nums')
#
# tmp1 = grouped.rolling(window = 3, min_periods = 3, center=True).mean()
#
#
# FM = feature_maker(h5_in)
# # tmp1, tmp2 = FM.rolling(10, 'std')
#
# tmp1, tmp2 = FM.shift(2)
#
#
#
# tmp1 = pd.DataFrame(np.random.rand(10, 5))
# kargs = {'min_periods':3, 'center':True}
# kargs = {}
# # kargs = (min_periods=3, center=True)
# # kargs = None
# df_rolling = tmp1.rolling(window = 3, **kargs)
#
# # df_rolling = tmp1.rolling(window=3, min_periods=3, center=True)
# df_rolling.mean().shift(0).shift(0).astype(np.float32)#
#
#
#
# if add_name_str is None:
#     add_name_str = 'shifted_'+str(shift_by)
#     add_name_str = 'feature_data_' + add_name_str
#
# key_exists = utils.h5_key_exists(h5_in, add_name_str)
#
#
#
# assert key_exists, "key name '"+ add_name_str + "' exists, if you want to overwrite it, delete it and re run this function"
# with h5py.File(h5_in, 'r+') as h:# can always change to save on each iteration to save ram but RAM should be fine
#     data_frame_in = pd.DataFrame(h['images'][:])
#     data_frame_out = shift_with_frame_nums(data_frame_in, h['frame_nums'], shift_by, add_name_str = add_name_str)
#     h.create_dataset_like(add_name_str, h['images'], data = data_frame_out.to_numpy())
#
#
#
# def shift_with_frame_nums(h5_in, shift_by, frame_nums=None, add_name_str=None, data_frame_in=None):
#     with h5py.File(h5_in, 'r+') as h:
#         if data_frame_in is None:
#             data_frame_in = h['images'][:]
#         if frame_nums is None:# this allows user to insert their own frame nums if needed
#             frame_nums = h['frame_nums'][:]
#         if add_name_str is None:
#             add_name_str = 'shifted_' + str(shift_by)
#         data_frame = data_frame_in.copy()
#         for i1, i2 in utils.loop_segments(frame_nums):
#             data_frame[i1:i2] = data_frame_in[i1:i2].shift(shift_by)
#         data_frame = data_frame.add_suffix('_' + add_name_str)
#     # return data_frame
#
#
# # def change_to_local_dir(bd):
# #     bd.split('Drive')[]
# #     bd2 = ''.join(bd.split('gdrive/My Drive'))
# #     return bd2
#
# tmp1 = Path(base_dir_all)
# tmp1 / 'asdf.txt'
#
# time_length = 1  # do not change we shift time in pandas dataframe now
# feature_len = 2048
# batch_size = 1000
# tmp_h5 = ''
# label_key = '[0, 1]- (no touch, touch)'
# G_test = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, tmp_h5,
#                                                        label_key=label_key, feature_len=feature_len)
#
# ####
# # In[35]:
#
#
# label_key = '[0, 1]- (no touch, touch)'
# label_key = 'labels'
# label_key in utils.print_h5_keys(tmp1, return_list=True, do_print=False)
#
# # In[79]:
#
#
# base_dir_all = r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\DATA_FULL'
# base_dir_all = r'G:\My Drive\Colab data\testing_TL_10_20_30\data_ANM234232_140120_AH1030_AAAA_a\3lag\ANM234232_140120_AH1030_AAAA_a_3lag___30'
# all_h5s = utils.get_h5s(base_dir_all)
# for h5 in all_h5s:
#     base_dir = os.path.dirname(h5) + os.sep + 'transformed_data'
#     df_tmp_x, tmp_y = make_data_chunks(h5, base_dir)
#
# # In[57]:
#
# utils.np_stats(y_pred)
# all_h5s[-3], h5
#
# # In[61]:
#
#
# # righthere
# alt_labels_h5s = r'G:\My Drive\colab_data2\model_testing\all_data\final_predictions\ALT_LABELS_FINAL_PRED'
# base_dir_all = r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\DATA_FULL'
#
# model_save_name_full = 'MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_PM_JC'
# # need to finish evaling above model so that num mod in h5 are equal
# # model_save_name_full = 'MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_ANDREW'
#
# model_save_name_full = 'model_5____' + '3lag__' + 'regular_labels__' + model_save_name_full + '_LIGHT_GBM'
# model_save_name_full
# all_h5s = utils.get_h5s(base_dir_all)
# alt_labels_h5s = utils.get_h5s(alt_labels_h5s)
# for h5, alt_labels_h5 in tqdm(zip(all_h5s, alt_labels_h5s)):
#     bn1 = os.path.basename(h5)
#     bn2 = os.path.basename(alt_labels_h5)
#     assert bn1[:16] == bn2[:16], 'files dont match'
#     base_dir = os.path.dirname(h5) + os.sep + 'transformed_data'
#     df_tmp_x, tmp_y = make_data_chunks(h5, base_dir)
#     df_tmp_x = concat_numpy_memmory_save(df_tmp_x, base_dir)
#     pred = bst.predict(df_tmp_x)
#     utils.np_stats(pred)
#
#     # is pred an int or a float in the first one it is an int64 which is weird
#     with h5py.File(alt_labels_h5, 'r+') as hf:
#         try:
#             hf.create_dataset(model_save_name_full, data=pred)
#         except:
#             del hf[model_save_name_full]
#             time.sleep(.1)
#             hf.create_dataset(model_save_name_full, data=pred)
#
# # In[32]:
#
#
# tmp1 = r'G:\My Drive\Colab data\testing_TL_10_20_30\data_ANM234232_140120_AH1030_AAAA_a\3lag\ANM234232_140120_AH1030_AAAA_a_3lag___30\train.h5'
# utils.print_h5_keys(tmp1)
# L = image_tools.get_h5_key_and_concatenate(tmp1, 'labels')
#
# img = image_tools.get_h5_key_and_concatenate(tmp1, 'images')
# utils.np_stats(L)
# img.shape, L.shape
#
# # In[41]:
#
#
# h5 = [r'D:\model_testing_features_data\feature_data\regular_80_border\ALL_RETRAIN_H5_data\3lag\train_3lag.h5',
#       r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\testing_TL_10_20_30\data_ANM234232_140120_AH1030_AAAA_a\3lag\ANM234232_140120_AH1030_AAAA_a_3lag___30\train.h5']
#
# #       r'D:\model_testing_features_data\feature_data\DATA_FULL_in_range_only\data_AH0667_170317_JC1241_AAAA\3lag\AH0667_170317_JC1241_AAAA_3lag.h5']
# base_dir = r'D:\temp_save\train_x_arrays_with_A'
# df_train_x, train_y = make_data_chunks(h5, base_dir)
#
# h5 = [r'D:\model_testing_features_data\feature_data\regular_80_border\ALL_RETRAIN_H5_data\3lag\val_3lag.h5',
#       r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\testing_TL_10_20_30\data_ANM234232_140120_AH1030_AAAA_a\3lag\ANM234232_140120_AH1030_AAAA_a_3lag___30\val.h5']
#
# #       r'D:\model_testing_features_data\feature_data\DATA_FULL_in_range_only\data_AH0698_170601_PM0121_AAAA\3lag\AH0698_170601_PM0121_AAAA_3lag.h5']
# base_dir = r'D:\temp_save\val_x_arrays_with_A'
# df_val_x, val_y = make_data_chunks(h5, base_dir)
#
# h5 = r'D:\model_testing_features_data\feature_data\regular_80_border\DATA_FULL\3lag\holy_test_set_10_percent_3lag.h5'
# base_dir = r'D:\temp_save\test_x_arrays'
# df_test_x, test_y = make_data_chunks(h5, base_dir)
#
# # In[34]:
#
#
# h5 = r'D:\model_testing_features_data\feature_data\8_testing_h5s\data_ANM234232_140118_AH1026_AAAA\3lag\ANM234232_140118_AH1026_AAAA_3lag.h5'
# base_dir = r'D:\temp_save\andrew_x_arrays'
# df_andrew_x, andrew_y = make_data_chunks(h5, base_dir)
#
# # In[4]:
#
#
# # utils.print_h5_keys(h5)
# # frame_nums_andrew = image_tools.get_h5_key_and_concatenate(h5, 'frame_nums')
# # frame_nums_andrew
#
#
# # ### load the data from file list
# #
#
# # In[42]:
#
#
# base_dir = r'D:\temp_save\train_x_arrays_with_A'
# df_train_x = concat_numpy_memmory_save(df_train_x, base_dir)
#
# base_dir = r'D:\temp_save\val_x_arrays_with_A'
# df_val_x = concat_numpy_memmory_save(df_val_x, base_dir)
#
# base_dir = r'D:\temp_save\test_x_arrays'
# df_test_x = concat_numpy_memmory_save(df_test_x, base_dir)
#
# # In[35]:
#
#
# base_dir = r'D:\temp_save\andrew_x_arrays'
# df_andrew_x = concat_numpy_memmory_save(df_andrew_x, base_dir)
#
#
# # In[43]:
#
#
# def rm_nan_rows(a, a2=None):
#     indx = ~np.isnan(a).any(axis=1)
#     if a2 is None:
#         return a[indx, :]
#     else:
#         return a[indx, :], a2[indx]
#
#
# # In[44]:
#
#
# df_train_x, train_y = rm_nan_rows(df_train_x, train_y)
# df_test_x, test_y, = rm_nan_rows(df_test_x, test_y)
# df_val_x, val_y = rm_nan_rows(df_val_x, val_y)
#
# # In[32]:
#
#
# df_andrew_x, andrew_y = rm_nan_rows(df_andrew_x, andrew_y)
#
# # #set up light GBM model
#
# # ###format data to lgb format
#
# # In[45]:
#
#
# train_data = lgb.Dataset(df_train_x, label=train_y)
# test_data = lgb.Dataset(df_test_x, label=test_y)
# validation_data = lgb.Dataset(df_val_x, label=val_y)
#
# # ###define params and metrics
#
# # In[4]:
#
#
# # param = {'objective': 'binary'}
# # param['metric'] = ['auc', 'binary_logloss']
# # param['device'] ='cpu'
# # param['max_bin'] ='63'
# # param['num_leaves'] =255
# # param['n_estimators'] = 400
# # param['early_stopping_rounds'] = 50
# # param['verbose'] = 1
#
# # param['num_round'] = 5000
#
#
# # In[46]:
#
#
# param = {'num_leaves': 255, 'objective': 'binary'}
# # param['metric'] = 'auc'
# param['metric'] = ['auc']
# param['early_stopping_rounds'] = 20
# param['device'] = 'cpu'
# param['verbose'] = 1
# param['n_estimators'] = 400
# param['num_round'] = 160
#
# # ###train
# #
#
# # In[47]:
#
#
# bst = lgb.LGBMClassifier(**param)
# # bst.get_params()
#
#
# # In[48]:
#
#
# st = time.time()
# bst.fit(df_train_x, train_y, eval_metric=['auc'], eval_set=[(df_val_x, val_y), (df_test_x, test_y)])
# str(datetime.timedelta(seconds=time.time() - st))
#
# # In[21]:
#
#
# # bst.save_model(r'D:\models\MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_PM_JC.txt')
#
#
# # In[49]:
#
#
# bst.booster_.save_model(
#     r'D:\models\MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_ANDREW.txt')
#
# # In[22]:
#
#
# utils.get_class_info(bst)
#
# # In[ ]:
#
#
# #
# bst = lgb.train(param, train_data, num_round, valid_sets=[validation_data], callbacks=callbacks)
#
# bst = lgb.LGBMClassifier(n_jobs=-1, n_estimators=400, **params)
#
# bst.fit(X_train, y_train, eval_metric='auc', eval_set=[(X_test, y_test)], early_stopping_rounds=50,
#         categorical_feature=categorical_features, verbose=5)
#
# # In[ ]:
#
#
# num_round = 2000
# bst = lgb.train(param, train_data, num_round, valid_sets=[validation_data])
#
# # In[ ]:
#
#
# # num_round = 5000
#
# callbacks = [lgb.early_stopping(50, first_metric_only=False, verbose=True, min_delta=0.0)]
#
# # In[ ]:
#
#
# # num_round = 3000
# # bst = lgb.train(param, train_data, num_round, valid_sets=[validation_data])
#
#
# # In[ ]:
#
#
# # visualize
# import matplotlib.pyplot as plt
# import matplotlib.style as style
# import seaborn as sns
# from matplotlib import pyplot
# from matplotlib.ticker import ScalarFormatter
#
# sns.set_context("talk")
# style.use('fivethirtyeight')
#
# fi = pd.DataFrame()
# fi['features'] = X.columns.values.tolist()
# fi['importance'] = model.feature_importance(importance_type="gain")
#
# sns.barplot(x='importance', y='features', data=fi.sort_values(by='importance', ascending=True))
#
# # In[ ]:
#
#
# bst.save_model('/content/gdrive/My Drive/colab_data2/model_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2.txt')  #
#
# # In[ ]:
#
#
# # utils.get_class_info(bst.params)
# bst.params
#
#
# # In[16]:
#
#
# def smooth_it(data, kernel_size=7):
#     kernel = np.ones(kernel_size) / kernel_size
#     return np.convolve(data, kernel, mode='same')
#
#
# # In[33]:
#
#
# yhat = bst.predict(df_andrew_x)
# np.mean(1 * (smooth_it(yhat, 7) > .5) == andrew_y)
#
# # In[17]:
#
#
# yhat = bst.predict(df_val_x)
# np.mean(1 * (smooth_it(yhat, 7) > .5) == val_y)
#
# # In[18]:
#
#
# yhat = bst.predict(df_train_x)
# np.mean(1 * (smooth_it(yhat, 7) > .5) == train_y)
#
# # In[19]:
#
#
# yhat = bst.predict(df_test_x)
# np.mean(1 * (smooth_it(yhat, 7) > .5) == test_y)
#
# # In[ ]:
#
#
# test_y
#
# # In[102]:
#
#
# utils.np_stats(yhat)
#
# # In[61]:
#
#
# plt.figure(figsize=(10, 10))
# real_y = andrew_y
#
# i1 += 4000
# # i1 = 0
# i2 = i1 + 4000
# yhat2 = 1 * (yhat > 0.5)
#
# # plt.plot(yhat[i1:i2])
# tmpy = smooth_it(yhat[i1:i2], kernel_size=3)
# plt.plot(tmpy, '--')
# tmpy = 1 * (tmpy > 0.5)
# plt.plot(tmpy)
# plt.plot(andrew_y[i1:i2] - 1.1)
#
# plt.plot((andrew_y[i1:i2] - yhat2[i1:i2] - 1.1) / 2, '.')
# plt.title(str(i1) + ' to ' + str(i2))
#
# # In[71]:
#
#
# plt.figure(figsize=(20, 10))
# frame_segs = utils.loop_segments(frame_nums_andrew, returnaslist=True)
#
# # for i1, i2 in utils.loop_segments(frame_nums_andrew):
#
# # cnt = -1
# cnt += 1
# i1, i2 = frame_segs[0][cnt], frame_segs[1][cnt]
#
# yhat2 = 1 * (yhat > 0.5)
#
# # plt.plot(yhat[i1:i2])
# tmpy = smooth_it(yhat[i1:i2], kernel_size=3)
# plt.plot(tmpy, '--')
# tmpy = 1 * (tmpy > 0.5)
# plt.plot(tmpy)
# plt.plot(andrew_y[i1:i2] - 1.1)
#
# plt.plot((andrew_y[i1:i2] - yhat2[i1:i2] - 1.1) / 2, '.')
# plt.title(str(i1) + ' to ' + str(i2))
#
# # In[42]:
#
#
# frame_segs[0][cnt], frame_segs[1][cnt]
#
# # #load model
#
# # In[60]:
#
#
# get_ipython().run_cell_magic('time', '',
#                              "model_file = r'D:\\models\\MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_ANDREW.txt'\nmodel_file = r'D:\\models\\MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_PM_JC.txt'\n\n# bst = lgb.load(filename = model_file)\n\nbst = lgb.Booster(model_file=model_file)")
#
# # In[13]:
#
#
# utils.get_class_info(bst)
#
# # In[17]:
#
#
# get_ipython().run_cell_magic('time', '',
#                              'x = df_test_x[:4000, :]\ny_pred = bst.predict(x)\nutils.np_stats(y_pred)\nplt.plot(y_pred)')
#
# # In[12]:
#
#
# get_ipython().run_cell_magic('time', '', 'x = df_test_x[:10, :]\ny_pred = bst.predict_proba(x)')
#
# # #make and save predictions
#
# # In[ ]:
#
#
# h5src = '/content/gdrive/My Drive/colab_data2/model_testing_features_data/feature_data/8_testing_h5s/'
#
# h5 = utils.get_h5s(h5src)
#
# time_length = 1
# batch_size = None
# label_key = '[0, 1]- (no touch, touch)'
# feature_len = 2048
# label_index_to_lstm_len = None
# edge_value = np.nan
# remove_any_time_points_with_edges = True
#
# G_test = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, h5, label_key=label_key,
#                                                        feature_len=feature_len,
#                                                        label_index_to_lstm_len=label_index_to_lstm_len,
#                                                        edge_value=edge_value,
#                                                        remove_any_time_points_with_edges=remove_any_time_points_with_edges)
# frame_nums_test = image_tools.get_h5_key_and_concatenate(h5, 'frame_nums')
#
# # In[ ]:
#
#
# test_x, test_y = G_test.__getitem__(0)
#
# # In[ ]:
#
#
# df_test_x = pd.DataFrame(test_x)
# del test_x  # save RAM
#
# # In[ ]:
#
#
# df_test_x = transform_x_data(df_test_x, frame_nums_test)
#
# # In[ ]:
#
#
# with open('df_test_x_2.pkl', 'wb') as f:
#     pickle.dump(df_test_x, f)
#
# # In[ ]:
#
#
# with open('df_test_x_2.pkl', 'rb') as f:
#     df_test_x = pickle.load(f)
#
# # In[ ]:
#
#
# # df_test_x = df_test_x.to_numpy()
#
#
# # In[ ]:
#
#
# yhat = bst.predict(df_test_x[:1000])
#
# # In[ ]:
#
#
# df_test_x[:1000].shape
#
# # In[ ]:
#
#
# from whacc import utils, image_tools, transfer_learning, analysis, error_analysis
#
# # #_____
#
# # In[ ]:
#
#
# # In[21]:
#
#
# get_ipython().system('pip install xgboost -U')
#
# import xgboost
#
# print("xgboost", xgboost.__version__)
# from xgboost import XGBClassifier
#
# # utils.get_class_info(xgboost)
#
#
# # In[ ]:
#
#
# clf = xgb.XGBClassifier(use_label_encoder=False, max_depth=200, n_estimators=400, subsample=1, learning_rate=0.09,
#                         reg_lambda=0.1, reg_alpha=0.1, gamma=1)
#
# # In[ ]:
#
#
# param = dict()
# param['gpu_id'] = 0
# param['tree_method'] = 'gpu_hist'
#
# param = {
#     'objective': 'binary:logistic',
#     'tree_method': 'binary:logistic',
#     'gpu_id': 0,
#     'use_label_encoder': False
# }
# model = XGBClassifier(**param)
# # train_data = lgb.Dataset(df_train_x, label=train_y)
# # validation_data = lgb.Dataset(df_test_x, label=test_y)
# model.fit(df_train_x, train_y)
#
# # In[ ]:
#
#
# grid = {'max_depth': 10}
# clf = XGBClassifier()
# clf.max_depth
# clf.set_params(**grid)
# XGBClassifier(base_score=0.5, colsample_bylevel=1, colsample_bytree=1,
#               gamma=0, learning_rate=0.1, max_delta_step=0, max_depth=10,
#               min_child_weight=1, missing=None, n_estimators=100, nthread=-1,
#               objective='binary:logistic', reg_alpha=0, reg_lambda=1,
#               scale_pos_weight=1, seed=0, silent=True, subsample=1)
# clf.max_depth
#
# # In[ ]:
#
#
# xgbmodel = XGBClassifier(objective='binary:logistic', use_label_encoder=False)
#
# # In[ ]:
#
#
# x, y = G_train.__getitem__(0)
#
# xgbmodel.fit(x, y)
#
# xgbmodel.get_params()
#
# # In[ ]:
#
#
# time_length = 7
# batch_size = int(92213 / 10)
# print(batch_size)
# label_key = '[0, 1]- (no touch, touch)'
# feature_len = 2048
# label_index_to_lstm_len = 3
# edge_value = np.nanf
# remove_any_time_points_with_edges = True
#
# h5 = '/content/data/ALL_RETRAIN_H5_data____temp/ALL_RETRAIN_H5_data/3lag/train_3lag.h5'
# G_train = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, h5, label_key=label_key,
#                                                         feature_len=feature_len,
#                                                         label_index_to_lstm_len=label_index_to_lstm_len,
#                                                         edge_value=edge_value,
#                                                         remove_any_time_points_with_edges=remove_any_time_points_with_edges)
#
# # In[ ]:
#
#
# model = XGBClassifier(objective='binary:logistic', use_label_encoder=False, par)
#
# model.fit(x, y)
#
# # #_____
#
# # ##build model
#
# # In[ ]:
#
#
# model = XGBClassifier(objective='binary:logistic', use_label_encoder=False, n_estimators=500, subsample=.9,
#                       max_depth=10)
#
# # In[ ]:
#
#
# model.get_params()
#
# # In[ ]:
#
#
# del model
#
# # In[ ]:
#
#
# # ##fit model
#
# # In[ ]:
#
#
# import time
#
# # In[ ]:
#
#
# st = time.time()
# x, y = G_train.__getitem__(0)
# model.fit(x, y)
# print(time.time() - st)
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# train_x.size  # about .5 gb on ram
#
# # In[ ]:
#
#
# xgbPredictor = xgboost.XGBRegressor(**self.xgb_params)
# xgbPredictor.fit(Xs, ys)
#
# # This hack should only be used if tree_method == gpu_hist or gpu_exact
# if self.xgb_params['tree_method'][:3] == 'gpu':
#     with tempfile.TemporaryFile() as dump_file:
#         pickle.dump(xgbPredictor, dump_file)
#         dump_file.seek(0)
#         self.predictor_ = pickle.load(dump_file)
# else:
#     self.predictor_ = xgbPredictor
#
# # #full batch all at once
#
# # In[ ]:
#
#
# # time_length = 7
# # batch_size = int(92213/1)
# # print(batch_size)
# # label_key='[0, 1]- (no touch, touch)'
# # feature_len=2048
# # label_index_to_lstm_len=3
# # edge_value= -1
# # remove_any_time_points_with_edges=True
#
# # h5 = '/content/data/ALL_RETRAIN_H5_data____temp/ALL_RETRAIN_H5_data/3lag/train_3lag.h5'
# # G_train = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, h5, label_key=label_key, feature_len=feature_len,
# #                                       label_index_to_lstm_len=label_index_to_lstm_len, edge_value=edge_value,
# #                                       remove_any_time_points_with_edges = remove_any_time_points_with_edges)
#
#
# # In[ ]:
#
#
# parameters = {'base_score': 0.23,  # i think this should be set to mean of the labels it is the bias value
#               'booster': 'gbtree',
#               'colsample_bylevel': 1,
#               # "colsample_bylevel" is the fraction of features (randomly selected) that will be used in each node to train each tree.
#               'colsample_bynode': 1,
#               'colsample_bytree': 1,
#               # "colsample_bytree" is the fraction of features (randomly selected) that will be used to train each tree.
#               'enable_categorical': False,
#               'gamma': 0,
#               'gpu_id': -1,
#               'importance_type': None,
#               'interaction_constraints': '',
#               'learning_rate': [.2, .1, .05, .01],
#               'max_delta_step': 0,
#               'max_depth': [8, 12],
#               'min_child_weight': 1,
#               'monotone_constraints': '()',
#               'n_estimators': [100, 250, 500],
#               'n_jobs': -1,  # use all core I think 8
#               'num_parallel_tree': 1,
#               'objective': 'binary:logistic',
#               'predictor': 'auto',
#               'random_state': 0,
#               'reg_alpha': 0,
#               'reg_lambda': 1,
#               'scale_pos_weight': 1,
#               'single_precision_histogram': True,  # supposed to make it faster as well
#               'subsample': 0.9,
#               # "subsample" is the fraction of the training samples (randomly selected) that will be used to train each tree.
#               #  this seems like this should not be 1 it should be lower to work better ......
#               'tree_method': 'hist',  # was 'exact' but 'hist' is supposed ot be faster
#               'use_label_encoder': False,
#               'validate_parameters': 1,
#               'verbosity': 1}
# parameters = {
#     'learning_rate': [.2, .1, .05, .01],
#     'max_depth': [8, 12],
#     'n_estimators': [100, 250, 500]}
# for k in parameters:
#     parameters[k] = utils.make_list(parameters[k], suppress_warning=True)
# # parameters
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # !lscpu
#
#
# # In[ ]:
#
#
# # # model = XGBClassifier(objective='binary:logistic', use_label_encoder=False, single_precision_histogram=True)
# # from sklearn.model_selection import GridSearchCV
# # # parameters = {
# # #     'max_depth': range (2, 10, 1),
# # #     'n_estimators': range(60, 220, 40),
# # #     'learning_rate': [0.1, 0.01, 0.05]}
#
# # model = XGBClassifier(objective= 'binary:logistic')
#
# # grid_search = GridSearchCV(
# #   estimator=model,
# #   param_grid=parameters,
# #   scoring = 'roc_auc',
# #   n_jobs = 10,
# #   cv = 10,
# #   verbose=True)
#
# # grid_search.fit(x[:20], y[:20])
#
#
# # In[ ]:
#
#
# from sklearn.model_selection import GridSearchCV
#
# model = XGBClassifier(objective='binary:logistic')
# kfold = 5
# gs = GridSearchCV(estimator=model,
#                   param_grid=parameters,
#                   scoring='roc_auc',
#                   n_jobs=-1,
#                   cv=kfold,
#                   verbose=3,
#                   return_train_score=True)
#
# # In[ ]:
#
#
# train_x, train_y
# test_x, test_y
#
# # In[ ]:
#
#
# # grid_result.best_estimator_.model.model.history.history
# from sklearn import preprocessing
#
# train_x = preprocessing.scale(train_x)
# test_x = preprocessing.scale(test_x)
# # train_x = preprocessing.scale(train_x)
# # train_x = preprocessing.scale(train_x)
#
#
# # In[ ]:
#
#
# train_x[:10, :5]
#
# # In[ ]:
#
#
# grid_result = gs.fit(train_x[:10, :5],
#                      train_y[:10],
#                      verbose=3)
#
# # In[ ]:
#
#
# grid_result = gs.fit([train_x[:10, :5]],
#                      train_y[:10],
#                      verbose=3)
#
# # In[ ]:
#
#
# # grid_result = gs.fit(train_x[:100, :5],
# #                     train_y[:100],
# #                     verbose=1,
# #                     validation_data=(test_x[:100, 5], test_y[:100]))
#
#
# # In[ ]:
#
#
# import joblib
#
# try:
#     joblib.dump(gs, '/content/gdrive/My Drive/XGBoost_gris_search/XGBoost_grid_search_001.pkl')
# except:
#     joblib.dump(grid_result, '/content/gdrive/My Drive/XGBoost_gris_search/XGBoost_grid_search_001.pkl')
# joblib.dump(grid_result, '/content/gdrive/My Drive/XGBoost_gris_search/XGBoost_grid_search_001.pkl')
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# clf = XGBClassifier()
# clf.max_depth
# clf.set_params(**grid)
#
# clf.get_params()
# history = clf.fit(train_x[:10, :5], train_y[:10],
#
#                   # In[ ]:
#
#                   grid={'max_depth': 10}
# clf = XGBClassifier()
# clf.max_depth
# clf.set_params(**grid)
# XGBClassifier(base_score=0.5,
#               colsample_bylevel=1,
#               colsample_bytree=1,
#               gamma=0,
#               learning_rate=0.1,
#               max_delta_step=0,
#               max_depth=10,
#               min_child_weight=1,
#               missing=None,
#               n_estimators=500,
#               nthread=-1,
#               objective='binary:logistic',
#               reg_alpha=0,
#               reg_lambda=1,
#               scale_pos_weight=1,
#               seed=0,
#               silent=True,
#               subsample=1)
# clf.n_estimators
# # clf.fit(x[:20], y[:20])
# history = clf.fit(train_x[:10, :5], train_y[:10])
#
# # In[ ]:
#
#
# grid = {'max_depth': 10}
# clf = XGBClassifier()
# clf.max_depth
# clf.set_params(**grid)
# XGBClassifier(base_score=0.5,
#               colsample_bylevel=1,
#               colsample_bytree=1,
#               gamma=0,
#               learning_rate=0.1,
#               max_delta_step=0,
#               max_depth=10,
#               min_child_weight=1,
#               missing=None,
#               n_estimators=500,
#               nthread=-1,
#               objective='binary:logistic',
#               reg_alpha=0,
#               reg_lambda=1,
#               scale_pos_weight=1,
#               seed=0,
#               silent=True,
#               subsample=1)
# clf.n_estimators
# # clf.fit(x[:20], y[:20])
# history = clf.fit(train_x[:10, :5], train_y[:10])
#
# # In[ ]:
#
#
# import time
#
# st = time.time()
#
# model = XGBClassifier(objective='binary:logistic', use_label_encoder=False, single_precision_histogram=True,
#                       n_estimators=500, subsample=.9, max_depth=10)
# x, y = G_train.__getitem__(0)
# model.fit(x, y)
#
# print(time.time() - st)
#
# # In[ ]:
#
#
# import time
#
# st = time.time()
#
# model_small_test = XGBClassifier(objective='binary:logistic', use_label_encoder=False, single_precision_histogram=True,
#                                  n_estimators=500, subsample=.9, max_depth=10)
# x, y = G_train.__getitem__(0)
# model_small_test.fit(x[:10, :5], y[:10])
#
# print(time.time() - st)
#
# # In[ ]:
#
#
# x[:10, :10]
#
# # In[ ]:
#
#
# from sklearn import preprocessing
#
# x = preprocessing.scale(x)
#
# # In[ ]:
#
#
# num_data_points = 10000
# model_small_test = XGBClassifier(objective='binary:logistic', use_label_encoder=False, single_precision_histogram=True,
#                                  n_estimators=500, subsample=.9, max_depth=10)
# st = time.time()
# model_small_test.fit(x[:num_data_points, :], y[:num_data_points])
# print(time.time() - st)
#
# # In[ ]:
#
#
# """
# before scaling all 2048 features
# 100  --> 5.22572660446167
# 1000 --> 11.855197429656982
# 10000--> 55.457772970199585
#
# after scaling all 2048 features
# 100  --> 3.963913917541504
# 1000 --> 9.200135469436646
# 10000--> 48.088958740234375
# """
#
# # In[ ]:
#
#
# xtmp = [100, 1000, 10000]
# plt.plot(xtmp, [5.22572660446167,
#                 11.855197429656982,
#                 55.457772970199585])
# plt.plot(xtmp, [3.963913917541504,
#                 9.200135469436646,
#                 48.088958740234375])
#
# # In[ ]:
#
#
# import pickle
#
# file_name = "/content/gdrive/My Drive/XGBoost_gris_search/xgb_reg.pkl"
#
# # save
# pickle.dump(model, open(file_name, "wb"))
#
# # In[ ]:
#
#
# model = pickle.load(open(file_name, "rb"))
#
# # In[ ]:
#
#
# from xgboost import plot_importance, plot_tree
#
# # In[ ]:
#
#
# # plt.plot(x[:, 163], x[:, 171], '.')
# tmpx = x[:10, :5]
# R1 = np.corrcoef(x)
# R1
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# plt.rcParams["figure.figsize"] = (30, 30)
# _ = plot_importance(model, height=0.9, max_num_features=200)
#
# # In[ ]:
#
#
# from IPython.core.pylabtools import figsize
#
# plot_importance(figsize=(10, 10))
#
# # In[ ]:
#
#
# real = np.asarray([])
# pred = np.asarray([])
# for kk in tqdm(range(G_val.__len__())):
#     x, y = G_val.__getitem__(kk)
# real = np.append(real, y)
# y_hat = model.predict(x)
# pred = np.append(pred, y_hat)
# np.mean(pred == real)
#
# # In[ ]:
#
#
# model
#
# # In[ ]:
#
#
# real = np.asarray([])
# pred = np.asarray([])
# for kk in tqdm(range(G_test.__len__())):
#     x, y = G_test.__getitem__(kk)
# real = np.append(real, y)
# y_hat = model.predict(x)
# pred = np.append(pred, y_hat)
# np.mean(pred == real)
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# real = np.asarray([])
# pred = np.asarray([])
# for kk in tqdm(range(G_train.__len__())):
#     x, y = G_train.__getitem__(kk)
# real = np.append(real, y)
# y_hat = model.predict(x)
# pred = np.append(pred, y_hat)
# np.mean(pred == real)
#
# # #__
# #
#
# #
#
# # In[ ]:
#
#
# utils.get_class_info(xgbmodel)
#
# # In[ ]:
#
#
# from xgboost import XGBClassifier
#
# xgbmodel = XGBClassifier(objective='binary:logistic')
#
# for k in range(G_train.__len__()):
#     x, y = G_train.__getitem__(k)
# xgbmodel.fit(x, y)
# for kk in range(G_val.__len__()):
#     x, y = G_val.__getitem__(kk)
# xgbmodel.score(x, y)
#
# # In[ ]:
#
#
# send_text('OOOOHHHHH SHIIIITTTTT ITS DOONNNEEEEE')
#
# # In[ ]:
#
#
# from xgboost import XGBClassifier
#
# utils.get_class_info(XGBClassifier)
# XGBClassifier()
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # #plot the features sorted to get an idea
#
# # In[ ]:
#
#
# h5 = '/content/data/ALL_RETRAIN_H5_data____temp/DATA_FULL/3lag/holy_test_set_10_percent_3lag.h5'
# # h5 = '/content/data/ALL_RETRAIN_H5_data____temp/ALL_RETRAIN_H5_data/3lag/train_3lag.h5'
# a = image_tools.ImageBatchGenerator_feature_array(7, 4000, h5, label_key='[0, 1]- (no touch, touch)', feature_len=2048,
#                                                   label_index_to_lstm_len=3, edge_value=-1,
#                                                   remove_any_time_points_with_edges=True)
# a.__len__()
#
# # In[ ]:
#
#
# x, y = a.__getitem__(9)
#
# # In[ ]:
#
#
# x.shape
#
# # In[ ]:
#
#
# # add_to = -2000
# add_to += 100
# plt.figure(figsize=(5, 700))
# tmp1 = int(2048 / 4)
# tmp1 = 2048 * 3
# # plt.imshow(x[2200+add_to:2350+add_to, ::tmp1].T)
# x2 = x[2200 + add_to:2350 + add_to, tmp1:tmp1 + 2000].T
# # sort_inds = np.argsort(np.mean(x2[:, 20:28], axis = 1))
# x3 = np.zeros_like(x2)
# for i, k in enumerate(sort_inds):
#     x3[i, :] = x2[k, :]
#
# y2 = y[2200 + add_to:2350 + add_to]
# y2 = y2 * np.max(x3)
# y2 = np.tile(y2, (10, 1))
# x3 = np.vstack((y2, x3, y2))
# plt.imshow(x3)
# # plt.axis('off')
# # plt.axis('tight')
# plt.show()
# # plt.figure(figsize=(5, 5))
# # time.sleep(3)
#
#
# plt.plot(y[2200 + add_to:2350 + add_to])
# plt.axis('off')
# plt.axis('tight')
# plt.ylim([-.1, 1.1])
# nums')
#
# grouped.rolling(10).mean()
#
#
# FM = feature_maker(h5_in)
# # tmp1, tmp2 = FM.rolling(10, 'std')
#
# tmp1, tmp2 = FM.shift(2)
#
#
#
#
#
#
# tmp1 = pd.DataFrame.rolling(window = 10)
#
#
#
#
# """
# pass function (or fucntion string)
# have frame nums and data self contained
# pass in k args
#
#
#
# """
#
#
# kargs = {'min_periods':3, 'center':True}
# # kargs = (min_periods=3, center=True)
# # kargs = None
# df_rolling = tmp1.rolling(window = 3, **kargs)
#
# # df_rolling = tmp1.rolling(window=3, min_periods=3, center=True)
# df_rolling.mean().shift(0).shift(0).astype(np.float32)#
#
#
#
# if add_name_str is None:
#     add_name_str = 'shifted_'+str(shift_by)
#     add_name_str = 'feature_data_' + add_name_str
#
# key_exists = utils.h5_key_exists(h5_in, add_name_str)
#
#
#
# assert key_exists, "key name '"+ add_name_str + "' exists, if you want to overwrite it, delete it and re run this function"
# with h5py.File(h5_in, 'r+') as h:# can always change to save on each iteration to save ram but RAM should be fine
#     data_frame_in = pd.DataFrame(h['images'][:])
#     data_frame_out = shift_with_frame_nums(data_frame_in, h['frame_nums'], shift_by, add_name_str = add_name_str)
#     h.create_dataset_like(add_name_str, h['images'], data = data_frame_out.to_numpy())
#
#
#
# def shift_with_frame_nums(h5_in, shift_by, frame_nums=None, add_name_str=None, data_frame_in=None):
#     with h5py.File(h5_in, 'r+') as h:
#         if data_frame_in is None:
#             data_frame_in = h['images'][:]
#         if frame_nums is None:# this allows user to insert their own frame nums if needed
#             frame_nums = h['frame_nums'][:]
#         if add_name_str is None:
#             add_name_str = 'shifted_' + str(shift_by)
#         data_frame = data_frame_in.copy()
#         for i1, i2 in utils.loop_segments(frame_nums):
#             data_frame[i1:i2] = data_frame_in[i1:i2].shift(shift_by)
#         data_frame = data_frame.add_suffix('_' + add_name_str)
#     # return data_frame
#
#
# # def change_to_local_dir(bd):
# #     bd.split('Drive')[]
# #     bd2 = ''.join(bd.split('gdrive/My Drive'))
# #     return bd2
#
# tmp1 = Path(base_dir_all)
# tmp1 / 'asdf.txt'
#
# time_length = 1  # do not change we shift time in pandas dataframe now
# feature_len = 2048
# batch_size = 1000
# tmp_h5 = ''
# label_key = '[0, 1]- (no touch, touch)'
# G_test = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, tmp_h5,
#                                                        label_key=label_key, feature_len=feature_len)
#
# ####
# # In[35]:
#
#
# label_key = '[0, 1]- (no touch, touch)'
# label_key = 'labels'
# label_key in utils.print_h5_keys(tmp1, return_list=True, do_print=False)
#
# # In[79]:
#
#
# base_dir_all = r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\DATA_FULL'
# base_dir_all = r'G:\My Drive\Colab data\testing_TL_10_20_30\data_ANM234232_140120_AH1030_AAAA_a\3lag\ANM234232_140120_AH1030_AAAA_a_3lag___30'
# all_h5s = utils.get_h5s(base_dir_all)
# for h5 in all_h5s:
#     base_dir = os.path.dirname(h5) + os.sep + 'transformed_data'
#     df_tmp_x, tmp_y = make_data_chunks(h5, base_dir)
#
# # In[57]:
#
# utils.np_stats(y_pred)
# all_h5s[-3], h5
#
# # In[61]:
#
#
# # righthere
# alt_labels_h5s = r'G:\My Drive\colab_data2\model_testing\all_data\final_predictions\ALT_LABELS_FINAL_PRED'
# base_dir_all = r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\DATA_FULL'
#
# model_save_name_full = 'MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_PM_JC'
# # need to finish evaling above model so that num mod in h5 are equal
# # model_save_name_full = 'MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_ANDREW'
#
# model_save_name_full = 'model_5____' + '3lag__' + 'regular_labels__' + model_save_name_full + '_LIGHT_GBM'
# model_save_name_full
# all_h5s = utils.get_h5s(base_dir_all)
# alt_labels_h5s = utils.get_h5s(alt_labels_h5s)
# for h5, alt_labels_h5 in tqdm(zip(all_h5s, alt_labels_h5s)):
#     bn1 = os.path.basename(h5)
#     bn2 = os.path.basename(alt_labels_h5)
#     assert bn1[:16] == bn2[:16], 'files dont match'
#     base_dir = os.path.dirname(h5) + os.sep + 'transformed_data'
#     df_tmp_x, tmp_y = make_data_chunks(h5, base_dir)
#     df_tmp_x = concat_numpy_memmory_save(df_tmp_x, base_dir)
#     pred = bst.predict(df_tmp_x)
#     utils.np_stats(pred)
#
#     # is pred an int or a float in the first one it is an int64 which is weird
#     with h5py.File(alt_labels_h5, 'r+') as hf:
#         try:
#             hf.create_dataset(model_save_name_full, data=pred)
#         except:
#             del hf[model_save_name_full]
#             time.sleep(.1)
#             hf.create_dataset(model_save_name_full, data=pred)
#
# # In[32]:
#
#
# tmp1 = r'G:\My Drive\Colab data\testing_TL_10_20_30\data_ANM234232_140120_AH1030_AAAA_a\3lag\ANM234232_140120_AH1030_AAAA_a_3lag___30\train.h5'
# utils.print_h5_keys(tmp1)
# L = image_tools.get_h5_key_and_concatenate(tmp1, 'labels')
#
# img = image_tools.get_h5_key_and_concatenate(tmp1, 'images')
# utils.np_stats(L)
# img.shape, L.shape
#
# # In[41]:
#
#
# h5 = [r'D:\model_testing_features_data\feature_data\regular_80_border\ALL_RETRAIN_H5_data\3lag\train_3lag.h5',
#       r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\testing_TL_10_20_30\data_ANM234232_140120_AH1030_AAAA_a\3lag\ANM234232_140120_AH1030_AAAA_a_3lag___30\train.h5']
#
# #       r'D:\model_testing_features_data\feature_data\DATA_FULL_in_range_only\data_AH0667_170317_JC1241_AAAA\3lag\AH0667_170317_JC1241_AAAA_3lag.h5']
# base_dir = r'D:\temp_save\train_x_arrays_with_A'
# df_train_x, train_y = make_data_chunks(h5, base_dir)
#
# h5 = [r'D:\model_testing_features_data\feature_data\regular_80_border\ALL_RETRAIN_H5_data\3lag\val_3lag.h5',
#       r'G:\My Drive\colab_data2\model_testing_features_data\feature_data\testing_TL_10_20_30\data_ANM234232_140120_AH1030_AAAA_a\3lag\ANM234232_140120_AH1030_AAAA_a_3lag___30\val.h5']
#
# #       r'D:\model_testing_features_data\feature_data\DATA_FULL_in_range_only\data_AH0698_170601_PM0121_AAAA\3lag\AH0698_170601_PM0121_AAAA_3lag.h5']
# base_dir = r'D:\temp_save\val_x_arrays_with_A'
# df_val_x, val_y = make_data_chunks(h5, base_dir)
#
# h5 = r'D:\model_testing_features_data\feature_data\regular_80_border\DATA_FULL\3lag\holy_test_set_10_percent_3lag.h5'
# base_dir = r'D:\temp_save\test_x_arrays'
# df_test_x, test_y = make_data_chunks(h5, base_dir)
#
# # In[34]:
#
#
# h5 = r'D:\model_testing_features_data\feature_data\8_testing_h5s\data_ANM234232_140118_AH1026_AAAA\3lag\ANM234232_140118_AH1026_AAAA_3lag.h5'
# base_dir = r'D:\temp_save\andrew_x_arrays'
# df_andrew_x, andrew_y = make_data_chunks(h5, base_dir)
#
# # In[4]:
#
#
# # utils.print_h5_keys(h5)
# # frame_nums_andrew = image_tools.get_h5_key_and_concatenate(h5, 'frame_nums')
# # frame_nums_andrew
#
#
# # ### load the data from file list
# #
#
# # In[42]:
#
#
# base_dir = r'D:\temp_save\train_x_arrays_with_A'
# df_train_x = concat_numpy_memmory_save(df_train_x, base_dir)
#
# base_dir = r'D:\temp_save\val_x_arrays_with_A'
# df_val_x = concat_numpy_memmory_save(df_val_x, base_dir)
#
# base_dir = r'D:\temp_save\test_x_arrays'
# df_test_x = concat_numpy_memmory_save(df_test_x, base_dir)
#
# # In[35]:
#
#
# base_dir = r'D:\temp_save\andrew_x_arrays'
# df_andrew_x = concat_numpy_memmory_save(df_andrew_x, base_dir)
#
#
# # In[43]:
#
#
# def rm_nan_rows(a, a2=None):
#     indx = ~np.isnan(a).any(axis=1)
#     if a2 is None:
#         return a[indx, :]
#     else:
#         return a[indx, :], a2[indx]
#
#
# # In[44]:
#
#
# df_train_x, train_y = rm_nan_rows(df_train_x, train_y)
# df_test_x, test_y, = rm_nan_rows(df_test_x, test_y)
# df_val_x, val_y = rm_nan_rows(df_val_x, val_y)
#
# # In[32]:
#
#
# df_andrew_x, andrew_y = rm_nan_rows(df_andrew_x, andrew_y)
#
# # #set up light GBM model
#
# # ###format data to lgb format
#
# # In[45]:
#
#
# train_data = lgb.Dataset(df_train_x, label=train_y)
# test_data = lgb.Dataset(df_test_x, label=test_y)
# validation_data = lgb.Dataset(df_val_x, label=val_y)
#
# # ###define params and metrics
#
# # In[4]:
#
#
# # param = {'objective': 'binary'}
# # param['metric'] = ['auc', 'binary_logloss']
# # param['device'] ='cpu'
# # param['max_bin'] ='63'
# # param['num_leaves'] =255
# # param['n_estimators'] = 400
# # param['early_stopping_rounds'] = 50
# # param['verbose'] = 1
#
# # param['num_round'] = 5000
#
#
# # In[46]:
#
#
# param = {'num_leaves': 255, 'objective': 'binary'}
# # param['metric'] = 'auc'
# param['metric'] = ['auc']
# param['early_stopping_rounds'] = 20
# param['device'] = 'cpu'
# param['verbose'] = 1
# param['n_estimators'] = 400
# param['num_round'] = 160
#
# # ###train
# #
#
# # In[47]:
#
#
# bst = lgb.LGBMClassifier(**param)
# # bst.get_params()
#
#
# # In[48]:
#
#
# st = time.time()
# bst.fit(df_train_x, train_y, eval_metric=['auc'], eval_set=[(df_val_x, val_y), (df_test_x, test_y)])
# str(datetime.timedelta(seconds=time.time() - st))
#
# # In[21]:
#
#
# # bst.save_model(r'D:\models\MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_PM_JC.txt')
#
#
# # In[49]:
#
#
# bst.booster_.save_model(
#     r'D:\models\MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_ANDREW.txt')
#
# # In[22]:
#
#
# utils.get_class_info(bst)
#
# # In[ ]:
#
#
# #
# bst = lgb.train(param, train_data, num_round, valid_sets=[validation_data], callbacks=callbacks)
#
# bst = lgb.LGBMClassifier(n_jobs=-1, n_estimators=400, **params)
#
# bst.fit(X_train, y_train, eval_metric='auc', eval_set=[(X_test, y_test)], early_stopping_rounds=50,
#         categorical_feature=categorical_features, verbose=5)
#
# # In[ ]:
#
#
# num_round = 2000
# bst = lgb.train(param, train_data, num_round, valid_sets=[validation_data])
#
# # In[ ]:
#
#
# # num_round = 5000
#
# callbacks = [lgb.early_stopping(50, first_metric_only=False, verbose=True, min_delta=0.0)]
#
# # In[ ]:
#
#
# # num_round = 3000
# # bst = lgb.train(param, train_data, num_round, valid_sets=[validation_data])
#
#
# # In[ ]:
#
#
# # visualize
# import matplotlib.pyplot as plt
# import matplotlib.style as style
# import seaborn as sns
# from matplotlib import pyplot
# from matplotlib.ticker import ScalarFormatter
#
# sns.set_context("talk")
# style.use('fivethirtyeight')
#
# fi = pd.DataFrame()
# fi['features'] = X.columns.values.tolist()
# fi['importance'] = model.feature_importance(importance_type="gain")
#
# sns.barplot(x='importance', y='features', data=fi.sort_values(by='importance', ascending=True))
#
# # In[ ]:
#
#
# bst.save_model('/content/gdrive/My Drive/colab_data2/model_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2.txt')  #
#
# # In[ ]:
#
#
# # utils.get_class_info(bst.params)
# bst.params
#
#
# # In[16]:
#
#
# def smooth_it(data, kernel_size=7):
#     kernel = np.ones(kernel_size) / kernel_size
#     return np.convolve(data, kernel, mode='same')
#
#
# # In[33]:
#
#
# yhat = bst.predict(df_andrew_x)
# np.mean(1 * (smooth_it(yhat, 7) > .5) == andrew_y)
#
# # In[17]:
#
#
# yhat = bst.predict(df_val_x)
# np.mean(1 * (smooth_it(yhat, 7) > .5) == val_y)
#
# # In[18]:
#
#
# yhat = bst.predict(df_train_x)
# np.mean(1 * (smooth_it(yhat, 7) > .5) == train_y)
#
# # In[19]:
#
#
# yhat = bst.predict(df_test_x)
# np.mean(1 * (smooth_it(yhat, 7) > .5) == test_y)
#
# # In[ ]:
#
#
# test_y
#
# # In[102]:
#
#
# utils.np_stats(yhat)
#
# # In[61]:
#
#
# plt.figure(figsize=(10, 10))
# real_y = andrew_y
#
# i1 += 4000
# # i1 = 0
# i2 = i1 + 4000
# yhat2 = 1 * (yhat > 0.5)
#
# # plt.plot(yhat[i1:i2])
# tmpy = smooth_it(yhat[i1:i2], kernel_size=3)
# plt.plot(tmpy, '--')
# tmpy = 1 * (tmpy > 0.5)
# plt.plot(tmpy)
# plt.plot(andrew_y[i1:i2] - 1.1)
#
# plt.plot((andrew_y[i1:i2] - yhat2[i1:i2] - 1.1) / 2, '.')
# plt.title(str(i1) + ' to ' + str(i2))
#
# # In[71]:
#
#
# plt.figure(figsize=(20, 10))
# frame_segs = utils.loop_segments(frame_nums_andrew, returnaslist=True)
#
# # for i1, i2 in utils.loop_segments(frame_nums_andrew):
#
# # cnt = -1
# cnt += 1
# i1, i2 = frame_segs[0][cnt], frame_segs[1][cnt]
#
# yhat2 = 1 * (yhat > 0.5)
#
# # plt.plot(yhat[i1:i2])
# tmpy = smooth_it(yhat[i1:i2], kernel_size=3)
# plt.plot(tmpy, '--')
# tmpy = 1 * (tmpy > 0.5)
# plt.plot(tmpy)
# plt.plot(andrew_y[i1:i2] - 1.1)
#
# plt.plot((andrew_y[i1:i2] - yhat2[i1:i2] - 1.1) / 2, '.')
# plt.title(str(i1) + ' to ' + str(i2))
#
# # In[42]:
#
#
# frame_segs[0][cnt], frame_segs[1][cnt]
#
# # #load model
#
# # In[60]:
#
#
# get_ipython().run_cell_magic('time', '',
#                              "model_file = r'D:\\models\\MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_ANDREW.txt'\nmodel_file = r'D:\\models\\MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_PM_JC.txt'\n\n# bst = lgb.load(filename = model_file)\n\nbst = lgb.Booster(model_file=model_file)")
#
# # In[13]:
#
#
# utils.get_class_info(bst)
#
# # In[17]:
#
#
# get_ipython().run_cell_magic('time', '',
#                              'x = df_test_x[:4000, :]\ny_pred = bst.predict(x)\nutils.np_stats(y_pred)\nplt.plot(y_pred)')
#
# # In[12]:
#
#
# get_ipython().run_cell_magic('time', '', 'x = df_test_x[:10, :]\ny_pred = bst.predict_proba(x)')
#
# # #make and save predictions
#
# # In[ ]:
#
#
# h5src = '/content/gdrive/My Drive/colab_data2/model_testing_features_data/feature_data/8_testing_h5s/'
#
# h5 = utils.get_h5s(h5src)
#
# time_length = 1
# batch_size = None
# label_key = '[0, 1]- (no touch, touch)'
# feature_len = 2048
# label_index_to_lstm_len = None
# edge_value = np.nan
# remove_any_time_points_with_edges = True
#
# G_test = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, h5, label_key=label_key,
#                                                        feature_len=feature_len,
#                                                        label_index_to_lstm_len=label_index_to_lstm_len,
#                                                        edge_value=edge_value,
#                                                        remove_any_time_points_with_edges=remove_any_time_points_with_edges)
# frame_nums_test = image_tools.get_h5_key_and_concatenate(h5, 'frame_nums')
#
# # In[ ]:
#
#
# test_x, test_y = G_test.__getitem__(0)
#
# # In[ ]:
#
#
# df_test_x = pd.DataFrame(test_x)
# del test_x  # save RAM
#
# # In[ ]:
#
#
# df_test_x = transform_x_data(df_test_x, frame_nums_test)
#
# # In[ ]:
#
#
# with open('df_test_x_2.pkl', 'wb') as f:
#     pickle.dump(df_test_x, f)
#
# # In[ ]:
#
#
# with open('df_test_x_2.pkl', 'rb') as f:
#     df_test_x = pickle.load(f)
#
# # In[ ]:
#
#
# # df_test_x = df_test_x.to_numpy()
#
#
# # In[ ]:
#
#
# yhat = bst.predict(df_test_x[:1000])
#
# # In[ ]:
#
#
# df_test_x[:1000].shape
#
# # In[ ]:
#
#
# from whacc import utils, image_tools, transfer_learning, analysis, error_analysis
#
# # #_____
#
# # In[ ]:
#
#
# # In[21]:
#
#
# get_ipython().system('pip install xgboost -U')
#
# import xgboost
#
# print("xgboost", xgboost.__version__)
# from xgboost import XGBClassifier
#
# # utils.get_class_info(xgboost)
#
#
# # In[ ]:
#
#
# clf = xgb.XGBClassifier(use_label_encoder=False, max_depth=200, n_estimators=400, subsample=1, learning_rate=0.09,
#                         reg_lambda=0.1, reg_alpha=0.1, gamma=1)
#
# # In[ ]:
#
#
# param = dict()
# param['gpu_id'] = 0
# param['tree_method'] = 'gpu_hist'
#
# param = {
#     'objective': 'binary:logistic',
#     'tree_method': 'binary:logistic',
#     'gpu_id': 0,
#     'use_label_encoder': False
# }
# model = XGBClassifier(**param)
# # train_data = lgb.Dataset(df_train_x, label=train_y)
# # validation_data = lgb.Dataset(df_test_x, label=test_y)
# model.fit(df_train_x, train_y)
#
# # In[ ]:
#
#
# grid = {'max_depth': 10}
# clf = XGBClassifier()
# clf.max_depth
# clf.set_params(**grid)
# XGBClassifier(base_score=0.5, colsample_bylevel=1, colsample_bytree=1,
#               gamma=0, learning_rate=0.1, max_delta_step=0, max_depth=10,
#               min_child_weight=1, missing=None, n_estimators=100, nthread=-1,
#               objective='binary:logistic', reg_alpha=0, reg_lambda=1,
#               scale_pos_weight=1, seed=0, silent=True, subsample=1)
# clf.max_depth
#
# # In[ ]:
#
#
# xgbmodel = XGBClassifier(objective='binary:logistic', use_label_encoder=False)
#
# # In[ ]:
#
#
# x, y = G_train.__getitem__(0)
#
# xgbmodel.fit(x, y)
#
# xgbmodel.get_params()
#
# # In[ ]:
#
#
# time_length = 7
# batch_size = int(92213 / 10)
# print(batch_size)
# label_key = '[0, 1]- (no touch, touch)'
# feature_len = 2048
# label_index_to_lstm_len = 3
# edge_value = np.nanf
# remove_any_time_points_with_edges = True
#
# h5 = '/content/data/ALL_RETRAIN_H5_data____temp/ALL_RETRAIN_H5_data/3lag/train_3lag.h5'
# G_train = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, h5, label_key=label_key,
#                                                         feature_len=feature_len,
#                                                         label_index_to_lstm_len=label_index_to_lstm_len,
#                                                         edge_value=edge_value,
#                                                         remove_any_time_points_with_edges=remove_any_time_points_with_edges)
#
# # In[ ]:
#
#
# model = XGBClassifier(objective='binary:logistic', use_label_encoder=False, par)
#
# model.fit(x, y)
#
# # #_____
#
# # ##build model
#
# # In[ ]:
#
#
# model = XGBClassifier(objective='binary:logistic', use_label_encoder=False, n_estimators=500, subsample=.9,
#                       max_depth=10)
#
# # In[ ]:
#
#
# model.get_params()
#
# # In[ ]:
#
#
# del model
#
# # In[ ]:
#
#
# # ##fit model
#
# # In[ ]:
#
#
# import time
#
# # In[ ]:
#
#
# st = time.time()
# x, y = G_train.__getitem__(0)
# model.fit(x, y)
# print(time.time() - st)
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# train_x.size  # about .5 gb on ram
#
# # In[ ]:
#
#
# xgbPredictor = xgboost.XGBRegressor(**self.xgb_params)
# xgbPredictor.fit(Xs, ys)
#
# # This hack should only be used if tree_method == gpu_hist or gpu_exact
# if self.xgb_params['tree_method'][:3] == 'gpu':
#     with tempfile.TemporaryFile() as dump_file:
#         pickle.dump(xgbPredictor, dump_file)
#         dump_file.seek(0)
#         self.predictor_ = pickle.load(dump_file)
# else:
#     self.predictor_ = xgbPredictor
#
# # #full batch all at once
#
# # In[ ]:
#
#
# # time_length = 7
# # batch_size = int(92213/1)
# # print(batch_size)
# # label_key='[0, 1]- (no touch, touch)'
# # feature_len=2048
# # label_index_to_lstm_len=3
# # edge_value= -1
# # remove_any_time_points_with_edges=True
#
# # h5 = '/content/data/ALL_RETRAIN_H5_data____temp/ALL_RETRAIN_H5_data/3lag/train_3lag.h5'
# # G_train = image_tools.ImageBatchGenerator_feature_array(time_length, batch_size, h5, label_key=label_key, feature_len=feature_len,
# #                                       label_index_to_lstm_len=label_index_to_lstm_len, edge_value=edge_value,
# #                                       remove_any_time_points_with_edges = remove_any_time_points_with_edges)
#
#
# # In[ ]:
#
#
# parameters = {'base_score': 0.23,  # i think this should be set to mean of the labels it is the bias value
#               'booster': 'gbtree',
#               'colsample_bylevel': 1,
#               # "colsample_bylevel" is the fraction of features (randomly selected) that will be used in each node to train each tree.
#               'colsample_bynode': 1,
#               'colsample_bytree': 1,
#               # "colsample_bytree" is the fraction of features (randomly selected) that will be used to train each tree.
#               'enable_categorical': False,
#               'gamma': 0,
#               'gpu_id': -1,
#               'importance_type': None,
#               'interaction_constraints': '',
#               'learning_rate': [.2, .1, .05, .01],
#               'max_delta_step': 0,
#               'max_depth': [8, 12],
#               'min_child_weight': 1,
#               'monotone_constraints': '()',
#               'n_estimators': [100, 250, 500],
#               'n_jobs': -1,  # use all core I think 8
#               'num_parallel_tree': 1,
#               'objective': 'binary:logistic',
#               'predictor': 'auto',
#               'random_state': 0,
#               'reg_alpha': 0,
#               'reg_lambda': 1,
#               'scale_pos_weight': 1,
#               'single_precision_histogram': True,  # supposed to make it faster as well
#               'subsample': 0.9,
#               # "subsample" is the fraction of the training samples (randomly selected) that will be used to train each tree.
#               #  this seems like this should not be 1 it should be lower to work better ......
#               'tree_method': 'hist',  # was 'exact' but 'hist' is supposed ot be faster
#               'use_label_encoder': False,
#               'validate_parameters': 1,
#               'verbosity': 1}
# parameters = {
#     'learning_rate': [.2, .1, .05, .01],
#     'max_depth': [8, 12],
#     'n_estimators': [100, 250, 500]}
# for k in parameters:
#     parameters[k] = utils.make_list(parameters[k], suppress_warning=True)
# # parameters
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # !lscpu
#
#
# # In[ ]:
#
#
# # # model = XGBClassifier(objective='binary:logistic', use_label_encoder=False, single_precision_histogram=True)
# # from sklearn.model_selection import GridSearchCV
# # # parameters = {
# # #     'max_depth': range (2, 10, 1),
# # #     'n_estimators': range(60, 220, 40),
# # #     'learning_rate': [0.1, 0.01, 0.05]}
#
# # model = XGBClassifier(objective= 'binary:logistic')
#
# # grid_search = GridSearchCV(
# #   estimator=model,
# #   param_grid=parameters,
# #   scoring = 'roc_auc',
# #   n_jobs = 10,
# #   cv = 10,
# #   verbose=True)
#
# # grid_search.fit(x[:20], y[:20])
#
#
# # In[ ]:
#
#
# from sklearn.model_selection import GridSearchCV
#
# model = XGBClassifier(objective='binary:logistic')
# kfold = 5
# gs = GridSearchCV(estimator=model,
#                   param_grid=parameters,
#                   scoring='roc_auc',
#                   n_jobs=-1,
#                   cv=kfold,
#                   verbose=3,
#                   return_train_score=True)
#
# # In[ ]:
#
#
# train_x, train_y
# test_x, test_y
#
# # In[ ]:
#
#
# # grid_result.best_estimator_.model.model.history.history
# from sklearn import preprocessing
#
# train_x = preprocessing.scale(train_x)
# test_x = preprocessing.scale(test_x)
# # train_x = preprocessing.scale(train_x)
# # train_x = preprocessing.scale(train_x)
#
#
# # In[ ]:
#
#
# train_x[:10, :5]
#
# # In[ ]:
#
#
# grid_result = gs.fit(train_x[:10, :5],
#                      train_y[:10],
#                      verbose=3)
#
# # In[ ]:
#
#
# grid_result = gs.fit([train_x[:10, :5]],
#                      train_y[:10],
#                      verbose=3)
#
# # In[ ]:
#
#
# # grid_result = gs.fit(train_x[:100, :5],
# #                     train_y[:100],
# #                     verbose=1,
# #                     validation_data=(test_x[:100, 5], test_y[:100]))
#
#
# # In[ ]:
#
#
# import joblib
#
# try:
#     joblib.dump(gs, '/content/gdrive/My Drive/XGBoost_gris_search/XGBoost_grid_search_001.pkl')
# except:
#     joblib.dump(grid_result, '/content/gdrive/My Drive/XGBoost_gris_search/XGBoost_grid_search_001.pkl')
# joblib.dump(grid_result, '/content/gdrive/My Drive/XGBoost_gris_search/XGBoost_grid_search_001.pkl')
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# clf = XGBClassifier()
# clf.max_depth
# clf.set_params(**grid)
#
# clf.get_params()
# history = clf.fit(train_x[:10, :5], train_y[:10],
#
#                   # In[ ]:
#
#                   grid={'max_depth': 10}
# clf = XGBClassifier()
# clf.max_depth
# clf.set_params(**grid)
# XGBClassifier(base_score=0.5,
#               colsample_bylevel=1,
#               colsample_bytree=1,
#               gamma=0,
#               learning_rate=0.1,
#               max_delta_step=0,
#               max_depth=10,
#               min_child_weight=1,
#               missing=None,
#               n_estimators=500,
#               nthread=-1,
#               objective='binary:logistic',
#               reg_alpha=0,
#               reg_lambda=1,
#               scale_pos_weight=1,
#               seed=0,
#               silent=True,
#               subsample=1)
# clf.n_estimators
# # clf.fit(x[:20], y[:20])
# history = clf.fit(train_x[:10, :5], train_y[:10])
#
# # In[ ]:
#
#
# grid = {'max_depth': 10}
# clf = XGBClassifier()
# clf.max_depth
# clf.set_params(**grid)
# XGBClassifier(base_score=0.5,
#               colsample_bylevel=1,
#               colsample_bytree=1,
#               gamma=0,
#               learning_rate=0.1,
#               max_delta_step=0,
#               max_depth=10,
#               min_child_weight=1,
#               missing=None,
#               n_estimators=500,
#               nthread=-1,
#               objective='binary:logistic',
#               reg_alpha=0,
#               reg_lambda=1,
#               scale_pos_weight=1,
#               seed=0,
#               silent=True,
#               subsample=1)
# clf.n_estimators
# # clf.fit(x[:20], y[:20])
# history = clf.fit(train_x[:10, :5], train_y[:10])
#
# # In[ ]:
#
#
# import time
#
# st = time.time()
#
# model = XGBClassifier(objective='binary:logistic', use_label_encoder=False, single_precision_histogram=True,
#                       n_estimators=500, subsample=.9, max_depth=10)
# x, y = G_train.__getitem__(0)
# model.fit(x, y)
#
# print(time.time() - st)
#
# # In[ ]:
#
#
# import time
#
# st = time.time()
#
# model_small_test = XGBClassifier(objective='binary:logistic', use_label_encoder=False, single_precision_histogram=True,
#                                  n_estimators=500, subsample=.9, max_depth=10)
# x, y = G_train.__getitem__(0)
# model_small_test.fit(x[:10, :5], y[:10])
#
# print(time.time() - st)
#
# # In[ ]:
#
#
# x[:10, :10]
#
# # In[ ]:
#
#
# from sklearn import preprocessing
#
# x = preprocessing.scale(x)
#
# # In[ ]:
#
#
# num_data_points = 10000
# model_small_test = XGBClassifier(objective='binary:logistic', use_label_encoder=False, single_precision_histogram=True,
#                                  n_estimators=500, subsample=.9, max_depth=10)
# st = time.time()
# model_small_test.fit(x[:num_data_points, :], y[:num_data_points])
# print(time.time() - st)
#
# # In[ ]:
#
#
# """
# before scaling all 2048 features
# 100  --> 5.22572660446167
# 1000 --> 11.855197429656982
# 10000--> 55.457772970199585
#
# after scaling all 2048 features
# 100  --> 3.963913917541504
# 1000 --> 9.200135469436646
# 10000--> 48.088958740234375
# """
#
# # In[ ]:
#
#
# xtmp = [100, 1000, 10000]
# plt.plot(xtmp, [5.22572660446167,
#                 11.855197429656982,
#                 55.457772970199585])
# plt.plot(xtmp, [3.963913917541504,
#                 9.200135469436646,
#                 48.088958740234375])
#
# # In[ ]:
#
#
# import pickle
#
# file_name = "/content/gdrive/My Drive/XGBoost_gris_search/xgb_reg.pkl"
#
# # save
# pickle.dump(model, open(file_name, "wb"))
#
# # In[ ]:
#
#
# model = pickle.load(open(file_name, "rb"))
#
# # In[ ]:
#
#
# from xgboost import plot_importance, plot_tree
#
# # In[ ]:
#
#
# # plt.plot(x[:, 163], x[:, 171], '.')
# tmpx = x[:10, :5]
# R1 = np.corrcoef(x)
# R1
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# plt.rcParams["figure.figsize"] = (30, 30)
# _ = plot_importance(model, height=0.9, max_num_features=200)
#
# # In[ ]:
#
#
# from IPython.core.pylabtools import figsize
#
# plot_importance(figsize=(10, 10))
#
# # In[ ]:
#
#
# real = np.asarray([])
# pred = np.asarray([])
# for kk in tqdm(range(G_val.__len__())):
#     x, y = G_val.__getitem__(kk)
# real = np.append(real, y)
# y_hat = model.predict(x)
# pred = np.append(pred, y_hat)
# np.mean(pred == real)
#
# # In[ ]:
#
#
# model
#
# # In[ ]:
#
#
# real = np.asarray([])
# pred = np.asarray([])
# for kk in tqdm(range(G_test.__len__())):
#     x, y = G_test.__getitem__(kk)
# real = np.append(real, y)
# y_hat = model.predict(x)
# pred = np.append(pred, y_hat)
# np.mean(pred == real)
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# real = np.asarray([])
# pred = np.asarray([])
# for kk in tqdm(range(G_train.__len__())):
#     x, y = G_train.__getitem__(kk)
# real = np.append(real, y)
# y_hat = model.predict(x)
# pred = np.append(pred, y_hat)
# np.mean(pred == real)
#
# # #__
# #
#
# #
#
# # In[ ]:
#
#
# utils.get_class_info(xgbmodel)
#
# # In[ ]:
#
#
# from xgboost import XGBClassifier
#
# xgbmodel = XGBClassifier(objective='binary:logistic')
#
# for k in range(G_train.__len__()):
#     x, y = G_train.__getitem__(k)
# xgbmodel.fit(x, y)
# for kk in range(G_val.__len__()):
#     x, y = G_val.__getitem__(kk)
# xgbmodel.score(x, y)
#
# # In[ ]:
#
#
# send_text('OOOOHHHHH SHIIIITTTTT ITS DOONNNEEEEE')
#
# # In[ ]:
#
#
# from xgboost import XGBClassifier
#
# utils.get_class_info(XGBClassifier)
# XGBClassifier()
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # In[ ]:
#
#
# # #plot the features sorted to get an idea
#
# # In[ ]:
#
#
# h5 = '/content/data/ALL_RETRAIN_H5_data____temp/DATA_FULL/3lag/holy_test_set_10_percent_3lag.h5'
# # h5 = '/content/data/ALL_RETRAIN_H5_data____temp/ALL_RETRAIN_H5_data/3lag/train_3lag.h5'
# a = image_tools.ImageBatchGenerator_feature_array(7, 4000, h5, label_key='[0, 1]- (no touch, touch)', feature_len=2048,
#                                                   label_index_to_lstm_len=3, edge_value=-1,
#                                                   remove_any_time_points_with_edges=True)
# a.__len__()
#
# # In[ ]:
#
#
# x, y = a.__getitem__(9)
#
# # In[ ]:
#
#
# x.shape
#
# # In[ ]:
#
#
# # add_to = -2000
# add_to += 100
# plt.figure(figsize=(5, 700))
# tmp1 = int(2048 / 4)
# tmp1 = 2048 * 3
# # plt.imshow(x[2200+add_to:2350+add_to, ::tmp1].T)
# x2 = x[2200 + add_to:2350 + add_to, tmp1:tmp1 + 2000].T
# # sort_inds = np.argsort(np.mean(x2[:, 20:28], axis = 1))
# x3 = np.zeros_like(x2)
# for i, k in enumerate(sort_inds):
#     x3[i, :] = x2[k, :]
#
# y2 = y[2200 + add_to:2350 + add_to]
# y2 = y2 * np.max(x3)
# y2 = np.tile(y2, (10, 1))
# x3 = np.vstack((y2, x3, y2))
# plt.imshow(x3)
# # plt.axis('off')
# # plt.axis('tight')
# plt.show()
# # plt.figure(figsize=(5, 5))
# # time.sleep(3)
#
#
# plt.plot(y[2200 + add_to:2350 + add_to])
# plt.axis('off')
# plt.axis('tight')
# plt.ylim([-.1, 1.1])



import h5py
import numpy as np
import cv2
import matplotlib.pyplot as plt
from tqdm import tqdm

h5_file = "/Users/phil/Dropbox/Autocurator/testing_data/MP4s/AH0667x170317_JON/AH0667x170317.h5"


trial_num = 1
frame_num = 0
methods = ['cv2.TM_CCOEFF', 'cv2.TM_CCOEFF_NORMED', 'cv2.TM_CCORR',
            'cv2.TM_CCORR_NORMED', 'cv2.TM_SQDIFF', 'cv2.TM_SQDIFF_NORMED']


methods = ['cv2.TM_SQDIFF', 'cv2.TM_SQDIFF_NORMED']


max_match_val_all_2 = ['cv2.TM_CCOEFF']
for method in methods:
    with h5py.File(h5_file, 'r+') as h:
        cum_frames = (h['trial_nums_and_frame_nums'][1, :]).astype('int').copy()
        cum_frames = np.cumsum(cum_frames)
        frame_ind = (np.sum(h['trial_nums_and_frame_nums'][1, :trial_num]) + frame_num).astype('int')
        print(frame_ind)
        template_img = h['images'][frame_ind].copy()
        max_match_val = []
        max_match_val_all = []
    # plt.figure()
    # plt.imshow(template_img)
    with h5py.File(h5_file, 'r+') as h:
        for i, k in enumerate(tqdm(h['images'])):
            if np.any(cum_frames == i):
                max_match_val_all.append(max_match_val)
                max_match_val = []
            res = cv2.matchTemplate(template_img,k,  eval(method))
            min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
            max_match_val.append(max_val)
            # max_match_val.append(res[0][0])
    max_match_val_all_2.append(max_match_val_all)
    plt.figure()
    plt.title(method)
    for k in max_match_val_all:
        plt.plot(k)



#
# with h5py.File(h5_file, 'r+') as h:
#     max_match_val_all = []
#     for i, k in enumerate(tqdm(h['images'])):
#         if np.any(cum_frames == i):
#             max_match_val_all.append(max_match_val)
#             max_match_val = []
#         a = np.sum(np.abs(template_img - k))
#         # res = cv2.matchTemplate(k, template_img, cv2.TM_CCOEFF)
#         # min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
#         max_match_val.append(a)
# plt.figure()
# plt.plot(max_match_val_all[0])
